!==================================================================================!
!
!  FD_CHECK
!
!==================================================================================!
SUBROUTINE fd_check
  USE global 
  USE mod_cart2D
  IMPLICIT NONE
  REAL :: dxmax, dxmin, vmax, dx
  REAL :: tmpvmax
  INTEGER :: I
  CHARACTER(LEN=8) :: rank_str

  write (rank_str,"(I3.3)") mpi_rank

  open(unit=420,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//rank_str(1:3)//'_stdout',status='old',position='append')

  write(420,*)'Begin Check:----:----::----:----::----:----::----:----::----:----::----:----::-'
  write(420,*)' '
  write(420,*)' seisfile     :  ',seisfile
  write(420,*)' modelfile    :  ',modelfile
  write(420,*)' recfile      :  ',recfile
  write(420,*)' nx           :  ',nx
  write(420,*)' nz           :  ',nz
  write(420,*)' time         :  ',stime
  write(420,*)' nt           :  ',nt
  write(420,*)' nop          :  ',nop

  ! min/max space increments
  !
  write(420,*)' dr(m)        :  ',dr

  ! angular increment
  !
  dxmax=rmax*dtheta
  dxmin=(rmax-dmax)*dtheta
  write(420,*)' max darc(m)  :  ',dxmax
  write(420,*)' min darc(m)  :  ',dxmin

  ! time increment - calculate dt from stability 
  !determine the maximum vmax of all ranks and store in 'vmax'
  vmax=maxval( sqrt(mu1/rho) )
  tmpvmax = vmax  !stores the local vmax
  CALL MPI_AllReduce(tmpvmax,vmax,1,MPI_REAL,MPI_MAX,MPI_COMM_WORLD,mpi_ierr)
  dx=min(dxmin,dxmax,dr)
  dt=stab* dx/(sqrt(2.)*vmax)

  write(420,*)' dt(s)        :  ',dt
  write(420,*)' izfree       :  ',izfree
  write(420,*)' igraves      :  ',igraves
  write(420,*)' attenuate    :  ',attenuate
  write(420,*)' '
  write(420,*)'Model-------------------------------------------------------------------------'
  write(420,*)' model_type   :  ',model_type
  write(420,*)' vs0(m/s)     :  ',vs0
  write(420,*)' rho0(kg/m3)  :  ',rho0
  write(420,*)' '
  write(420,*)'Source------------------------------------------------------------------------'
  write(420,*)' sdepth       :  ',sdepth
  write(420,*)' act.depth    :  ',rmax-r1(1,isz)
  write(420,*)' isz          :  ',isz
  write(420,*)' T_dom(s)     :  ',aa
  write(420,*)' it0          :  ',it0
  write(420,*)' '
  write(420,*)'Vs, Rho-----------------------------------------------------------------------'
  write(420,*)' Max(vs1)     :  ',maxval(sqrt(mu1/rho))
  write(420,*)' Min(vs1)     :  ',minval(sqrt(mu1/rho))
  write(420,*)' Max(vs2)     :  ',maxval(sqrt(mu2/rho))
  write(420,*)' Min(vs2)     :  ',minval(sqrt(mu2/rho))
  write(420,*)' Max(rho)     :  ',maxval(rho)
  write(420,*)' Min(rho)     :  ',minval(rho)
  write(420,*)' '
  write(420,*)'Stability---------------------------------------------------------------------'
  write(420,*)' Stability factor (<1) :  ',sqrt(2.)*vmax*dt/dx
  write(420,*)' '
  write(420,*)'R, Theta----------------------------------------------------------------------'
  write(420,*) minval(r1),' < r1 < ',maxval(r1)
  write(420,*) minval(r2),' < r2 < ',maxval(r2)
  write(420,*) minval(theta1),' < theta1 < ',maxval(theta1)
  write(420,*) minval(theta2),' < theta2 < ',maxval(theta2)
  write(420,*)' '
  write(420,*)'Physical boundaries------------------------------------------------------------'
  write(420,*)' Free surface r       : ',r1(1,izfree)
  write(420,*)' CMB          r       : ',r1(1,nz-izfree+1)
  write(420,*)' Left  boundary theta : ',theta1(izfree,1)
  write(420,*)' Right boundary theta : ',theta1(nx-izfree+1,1)
  write(420,*)' '
  write(420,*)'Output-------------------------------------------------------------------------'
  write(420,*)' iout            : ',iout
  write(420,*)' icheck          : ',icheck
  write(420,*)' tsnap           : ',tsnap
  write(420,*)' snapstart       : ',snapstart
  write(420,*)' sacout          : ',sacout
  write(420,*)' isnap           : ',isnap
  write(420,*)' '
  write(420,*)'Receivers----------------------------------------------------------------------'
  write(420,*)' izrec           : ',izrec
  write(420,*)' stdp            : ',(rmax-r1(1,izrec))/1000.
  write(420,*)' '
  write(420,*) nr ,' Receiver locations '
  do i=1,nr
    write(420,*)' Receiver ',i,' at theta = ',rec_loc(i),'deg'
  enddo
  write(420,*)' '
  write(420,*)'Begin Time Loop----------------------------------------------------------------'
  write(420,*)' Starting time loop for ...',nt,' time steps ... '
  write(420,*)' '

  write(420,*)'End Check:----:----::----:----::----:----::----:----::----:----::----:----::--:'
  write(420,*)' '
  close(420)

END SUBROUTINE fd_check
!==================================================================================!

