!==================================================================================!
!
!  MODULE:  DERIVATIVE
!
!==================================================================================!
MODULE derivative
  CONTAINS

  !==================================================================================!
  !
  !  PDER
  !
  !==================================================================================!
  SUBROUTINE pder(h,f,dx,nx_t,nz_t,dim,dir,nop_t)
  IMPLICIT NONE

  integer nx_t,nz_t,nop_t ! does not work on linux, necessary for hitachi ???

  real h(nx_t,nz_t)
  real f(nx_t,nz_t)
  integer, parameter :: maxnop=8
  real g(maxnop)
  integer dir,dim,i,j
  real dx
!  integer nx_t,nz_t,nop_t

  ! derivative operators
  !
  if(nop_t==4) then
    g(1)=.0416666666
    g(2)=-1.125
  endif

  if(nop_t==6) then
    g(1)=-0.00468750000000
    g(2)= 0.06510416666667
    g(3)=-1.17187500000000
  endif

  if(nop_t==8) then
    g(1)=0.1344048E-02
    g(2)=-0.1324718E-01
    g(3)=0.8948020E-01
    g(4)=-0.1211563E+01
  endif

  ! the symmetry
  !
  do i=nop_t/2+1,nop_t
    g(i) = -g(nop_t-i+1)
  enddo

  if (dim == 1) then
    if ( dir == -1 ) then              ! dir=-1
      if(nop_t==2) then
        forall (i=2:nx_t,j=1:nz_t)
          h(i,j)=(f(i,j)-f(i-1,j))/dx	
        end forall
      endif ! endif nop_t

      if(nop_t==4) then
        forall (i=3:nx_t-1,j=1:nz_t)
          h(i,j)=(g(1)*f(i-2,j)+  &
          g(2)*f(i-1,j)+  &
          g(3)*f(i  ,j)+  &
          g(4)*f(i+1,j))/dx 
        end forall
      endif ! endif nop_t

      if(nop_t==8) then
        forall (i=5:nx_t-3,j=1:nz_t)
          h(i,j)=(g(1)*f(i-4,j)+  &
          g(2)*f(i-3,j)+  &
          g(3)*f(i-2,j)+  &
          g(4)*f(i-1,j)+  &
          g(5)*f(i  ,j)+  &
          g(6)*f(i+1,j)+  &
          g(7)*f(i+2,j)+  &
          g(8)*f(i+3,j))/dx 
        end forall
      endif ! endif nop_t

    else ! else dir                    ! dir=+1
      if(nop_t==2) then
        forall (i=1:nx_t-1,j=1:nz_t)
          h(i,j)=(f(i+1,j)-f(i,j))/dx	
        end forall
      endif ! endif nop_t

      if(nop_t==4) then
        forall (i=2:nx_t-2,j=1:nz_t)
          h(i,j)=(g(1)*f(i-1,j)+ g(2)*f(i,j)+ g(3)*f(i+1,j)+ g(4)*f(i+2,j))/dx 
        end forall
      endif ! endif nop_t

      if(nop_t==8) then
        forall (i=4:nx_t-4,j=1:nz_t)
          h(i,j)=(g(1)*f(i-3,j)+ g(2)*f(i-2,j)+ g(3)*f(i-1,j)+ g(4)*f(i  ,j)+  &
                  g(5)*f(i+1,j)+ g(6)*f(i+2,j)+ g(7)*f(i+3,j)+ g(8)*f(i+4,j))/dx 
        end forall
      endif ! endif nop_t
    endif ! endif dir

! Heiner ended here!!

  else ! else dim
    if ( dir == -1 ) then
      if(nop_t==2) then
        forall (i=1:nx_t,j=2:nz_t)
          h(i,j)=(f(i,j)-f(i,j-1))/dx	
        end forall
      endif ! endif nop_t

      if(nop_t==4) then
        forall (i=1:nx_t,j=3:nz_t-1)
          h(i,j)=(g(1)*f(i,j-2)+ g(2)*f(i,j-1)+ g(3)*f(i,j)+ g(4)*f(i,j+1))/dx 
        end forall
      endif ! endif nop_t

      if(nop_t==8) then
        forall (i=1:nx_t,j=5:nz_t-3)
          h(i,j)=(g(1)*f(i,j-4)+ g(2)*f(i,j-3)+ g(3)*f(i,j-2)+ g(4)*f(i,j-1)+  &
                  g(5)*f(i,j)+ g(6)*f(i,j+1)+ g(7)*f(i,j+2)+ g(8)*f(i,j+3))/dx 
        end forall
      endif ! endif nop_t

    else ! else dir
      if(nop_t==2) then
        forall (i=1:nx_t,j=1:nz_t-1)
          h(i,j)=(f(i,j+1)-f(i,j))/dx	
        end forall
      endif ! endif nop_t

      if(nop_t==4) then
        forall (i=1:nx_t,j=2:nz_t-2)
          h(i,j)=(g(1)*f(i,j-1)+ g(2)*f(i,j)+ g(3)*f(i,j+1)+ g(4)*f(i,j+2))/dx 
        end forall
      endif ! endif nop_t

      if(nop_t==8) then
        forall (i=1:nx_t,j=4:nz_t-4)
          h(i,j)=(g(1)*f(i,j-3)+ g(2)*f(i,j-2)+ g(3)*f(i,j-1)+ g(4)*f(i,j)+  &
                  g(5)*f(i,j+1)+ g(6)*f(i,j+2)+ g(7)*f(i,j+3)+ g(8)*f(i,j+4))/dx 
        end forall
      endif ! endif nop_t
    endif ! endif dir
  endif ! endif dim
	
  END SUBROUTINE pder
  !==================================================================================!


  !==================================================================================!
  !
  !  INTER
  !
  !==================================================================================!
  SUBROUTINE inter(h,f,nx_t,nz_t,dim,dir)
  IMPLICIT NONE
  real h(nx_t,nz_t)
  real f(nx_t,nz_t)
  real dx
  integer nx_t,nz_t
  integer dir,dim,i,j

  if (dim == 1) then
    if ( dir == -1 ) then
      forall (i=2:nx_t-1,j=2:nz_t-1)
        h(i,j)=0.5*(f(i,j)+f(i-1,j))
      end forall

    else ! else dir
      forall (i=2:nx_t-1,j=2:nz_t-1)
        h(i,j)=0.5*(f(i+1,j)+f(i,j))
      end forall
    endif ! endif dir

  else ! else dim
    if ( dir == -1 ) then
      forall (i=2:nx_t-1,j=2:nz_t-1)
        h(i,j)=0.5*(f(i,j)+f(i,j-1))
      end forall

    else ! else dir
      forall (i=2:nx_t-1,j=2:nz_t-1)
        h(i,j)=0.5*(f(i,j+1)+f(i,j))
      end forall
    endif ! endif dir
  endif ! endif dim

  END SUBROUTINE inter
  !==================================================================================!

END MODULE derivative

