!==================================================================================!
!
!  FD_EVOLUTION
!
!==================================================================================!
! contains graves surface conditions (not yet working...)

include 'fd2_deriv.f90'  ! subroutines pder(), inter()

subroutine fd_evolution
  use derivative
  use global
  use mod_cart2D
  implicit none
  integer i,k
  real, dimension (nx,nz) :: upi,erp,etp,rtemp
  real :: alpha,delta,gamma ! constants for graves surface condition

  character(8) :: rank_str ! string for file name
  write (rank_str,"(I3.3)") mpi_rank

  ! 1.  Global Checks
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  !if(mpi_rank==0) then
    if(mod(it,icheck)==0) then
      open(unit=420,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//rank_str(1:3)//'_stdout',status='old',position='append')
      write(420,*)' Values at it = ',it, '/', nt
      write(420,*)'rank ',mpi_rank,' ',minval(up), ' < u_p < ', maxval(up)
      write(420,*)'rank ',mpi_rank,' ',minval(srp),' < s_rp < ',maxval(srp)
      write(420,*)'rank ',mpi_rank,' ',minval(stp),' < s_tp < ',maxval(stp)
      close(420)
    endif
  !endif
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  !Apply taper to velocity
  !-----------------------
  up = window * up 
  
  ! 2. Free Surface - Stress 
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(mpi_zrank==0) then ! this is always TRUE
    forall (i=1:izfree-1)
      srp(:,izfree-i)      = -srp(:,izfree-1+i)
      srp(:,nz-izfree+i)   = -srp(:,nz-izfree+1-i)
      stp(:,izfree-i)      = -stp(:,izfree+i)
      stp(:,nz-izfree+1+i) = -stp(:,nz-izfree+1-i)
    end forall
  endif
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 3. Symmetries at Left and Right Boundaries - Stress
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(mpi_xrank==0) then                 ! symmetry at theta=0
    srp(izfree,:)=0.
    do i=1,izfree-1
      stp(izfree-i,:)=-stp(izfree-1+i,:)
      srp(izfree-i,:)=srp(izfree+i,:)
    enddo
  endif

  if(mpi_xrank==dim_size(MPI_X)-1) then ! symmetry at theta=PI
    srp(nx-izfree-BSIZE+1,:)=0.
    do i=1,izfree+BSIZE-1
      stp(nx-izfree-BSIZE+i,:)=-stp(nx-izfree-BSIZE+1-i,:)
      srp(nx-izfree-BSIZE+1+i,:)=srp(nx-izfree-BSIZE+1-i,:)
    enddo
  endif
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 4. Update Velocity Field
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  call pder(rtemp,srp,dr,nx,nz,2,-1,nop)
  upi=rtemp
  call inter(rtemp,srp,nx,nz,2,-1)
  upi=upi+1./r1*3.*rtemp

  call pder(rtemp,stp,dtheta,nx,nz,1,-1,nop)
  upi=upi+1./r1*rtemp
  call inter(rtemp,stp,nx,nz,1,-1)
  upi=upi+1./r1*cot1*2.*rtemp

  !upi=(upi/rho) * window ! apply window here?  ???? -> tested
                          ! doesn't make a bit of difference to the output
                          ! if we apply the window here or not.
  upi=upi/rho
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 5. Add Source - Body Force
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(mpi_xrank==0) then ! currently, only sources in first rank allowed
    if(source_type==1) then
      upi(izfree+1,isz)=upi(izfree+1,isz)+so(it)
    endif
  endif
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:


  ! 6. Time Evolution
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  up=up+upi*dt
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:


  ! 7. Free Surface - Velocity
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(mpi_zrank==0) then
    if(igraves==0) then
      forall(i=1:izfree-1) 
        up(:,izfree-i)=up(:,izfree+i)
        up(:,nz-izfree+1+i)=up(:,nz-izfree+1-i)
      end forall
    else if (igraves==1) then 
      ! graves method (unfinished):
      ! check and finish this (Fr. 21.6.02)
      ! Graves formular from Tarje (yet unverified):
      do i=1,izfree-1
        alpha= dr/( r1(1,izfree)-dr/2. )
        delta= -(alpha + 1./(1.+1./alpha))
        gamma=1. -alpha/2

        up(:,izfree-i)     =gamma*up(:,izfree-i+2)   +delta*up(:,izfree-i+1);
        up(:,nz-izfree+1+i)=gamma*up(:,nz-izfree+1+i-2)+delta*up(:,nz-izfree+1+i-1)
      enddo
      ! end graves method
    endif ! endif igraves
  endif ! endif mpi_zrank
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:


  ! 8. Symmetries at Left and Right Boundaries - Velocity
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(mpi_xrank==0) then ! symmetry at theta=0
    up(izfree,:)=0.
    do i=1,izfree-1
      up(izfree-i,:)=-up(izfree+i,:)
    enddo
  endif

  if(mpi_xrank==dim_size(MPI_X)-1) then ! symmetry at theta=PI
    up(nx-izfree-BSIZE+1,:)=0.
    do i=1,izfree+BSIZE-1
      up(nx-izfree-BSIZE+1+i,:)=-up(nx-izfree-BSIZE+1-i,:)
    enddo
  endif
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 9. Communicate Velocity Across Ranks
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  call mpi_communicate(up)
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 10. Update Strain
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  IF (attenuate == 0) THEN
    call pder(rtemp,up,dr,nx,nz,2,1,nop)
    erp=rtemp
    call inter(rtemp,up,nx,nz,2,1)
    erp=erp-1./r2*rtemp
    erp=mu2*erp

    call  pder(rtemp,up,dtheta,nx,nz,1,1,nop)
    etp=1./r1*rtemp
    call  inter(rtemp,up,nx,nz,1,1)
    etp=etp-1./r1*rtemp*cot2
    etp=mu1*etp
  ELSEIF (attenuate == 1) THEN
    call pder(rtemp,up,dr,nx,nz,2,1,nop)
    erp=rtemp
    call inter(rtemp,up,nx,nz,2,1)
    erp=erp-1./r2*rtemp
    erp=0.5*erp

    call  pder(rtemp,up,dtheta,nx,nz,1,1,nop)
    etp=1./r1*rtemp
    call  inter(rtemp,up,nx,nz,1,1)
    etp=etp-1./r1*rtemp*cot2
    etp=0.5*etp
  ENDIF
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 11. Update Stress
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  IF (attenuate == 0) THEN
    srp=(srp+erp*dt) * window
    stp=(stp+etp*dt) * window
    !srp=(srp+erp*dt)
    !stp=(stp+etp*dt)
  ELSEIF (attenuate == 1) THEN

    rtemp = 0.0
    DO k=1,Nk
      rtemp(1:nx,1:nz) = rtemp(1:nx,1:nz) + mvr1(1:nx,1:nz,k)
    ENDDO
    !srp = srp + 2.0*Mu_1*(erp - rtemp)*dt  
    srp = (srp + 2.0*Mu_1*(erp - rtemp)*dt) * window 

    rtemp = 0.0
    DO k=1,Nk
      rtemp(1:nx,1:nz) = rtemp(1:nx,1:nz) + mvr2(1:nx,1:nz,k)
    ENDDO
    !stp = stp + 2.0*Mu_2*(etp - rtemp)*dt  
    stp = (stp + 2.0*Mu_2*(etp - rtemp)*dt) * window  

  ENDIF

  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! add source(s)
  !
  !if(mpi_xrank==0) then ! currently, only sources in first rank allowed
  !  if(source_type==2)then
  !    srp(izfree+1,isz)=srp(izfree+1,isz)+so(it)
  !  elseif(source_type==3)then
  !    stp(izfree+1,isz)=stp(izfree+1,isz)+so(it)
  !  endif
  !endif

  ! 12.  Communicate Stress Across Ranks
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  call mpi_communicate(srp)
  call mpi_communicate(stp)
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  ! 13. Update Memory Variables
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  IF (attenuate == 1) THEN
    DO k=1,Nk
      mvr1(1:nx,1:nz,k) = exp((-dt/rlxt(k)))*mvr1(1:nx,1:nz,k) + &
          & wtr1(1:nx,1:nz,k)*(1.0 - exp((-dt/rlxt(k))))*erp

      mvr2(1:nx,1:nz,k) = exp((-dt/rlxt(k)))*mvr2(1:nx,1:nz,k) + &
          & wtr2(1:nx,1:nz,k)*(1.0 - exp((-dt/rlxt(k))))*etp
    ENDDO
  ENDIF

END SUBROUTINE fd_evolution
!==================================================================================!

