
subroutine init_graphics(gif_mode,nhoriz,nvert)
  use plot_mod             ! pgplot module
  use global, only: seisfile
  implicit none
  integer gif_mode,nhoriz,nvert
  if(gif_mode==0) then; plot_device='/xw'
  else;            plot_device='pics/bild_#.gif/gif'
  endif

  ! init with four panels
  !call plot_init(plot_device,8.,1.,2,2,font=1)

  ! init with one panel
  call plot_init(plot_device,8.,1.,nhoriz,nvert,font=1)
end subroutine init_graphics

! subroutine map_to_disk
! maps a domain matrix to a cartesian canvas.
!
! parameter:
!    dst_matrix  Matrix to plot
!    dst_x       xsize of dst_matrix
!    dst_z       zsize of dst_matrix
!
!    src         Domain matrix with values to insert into dst_matrix
!    src_x       xsize of src
!    src_z       zsize of src
!
!    level       corresponding depth range of matrix "src"
!                level=1 : uppermost matrix (radial range=from surface to 0.5 Earth radii)
!                level=2 : next matrix (radial range= 0.5 to 0.25 Earth radii)
!                level=3 : etc.
!    maxlevel    total number of domains in simulation
!
subroutine circle_to_disk(dst_matrix,dst_x,dst_z,radius)
    ! use global
    implicit none
    real, parameter :: pi=3.14159265358
    integer dst_x,dst_z,src_x,src_z
    real radius,color
    real dst_matrix(dst_x,dst_z) ! canvas for plotting

    integer i,ix,iz
    real maxampl,angle
    maxampl=maxval(abs(dst_matrix))

    if(radius<0.) then; color=-.3; else; color=+.3;endif

    do i=0,200-1
      angle=i*pi/200.
      ix= nint(cos(angle)*abs(radius)*dst_x/2)
      iz= nint(sin(angle)*abs(radius)*dst_z/2)
      dst_matrix(dst_x/2+ix,dst_z/2+iz)=maxampl*color
      dst_matrix(dst_x/2+ix,dst_z/2-iz)=maxampl*color
    enddo
end subroutine circle_to_disk

subroutine map_to_disk(dst_matrix,dst_x,dst_z,src,src_x,src_z)
  use global, only : Rmax, Dmax
  implicit none
  real, parameter :: pi=3.14159265358
  integer dst_x,dst_z,src_x,src_z
  real dst_matrix(dst_x,dst_z) ! canvas for plotting
  real src(src_x,src_z)        ! matrix segment to plot into canvas
  real rx,rz                   ! normalized polar koordinates
  integer px,pz                ! pixel indices
  integer level,maxlevel
  real theta,radi
  real seg_theta1,seg_theta2   ! start,end angle of segment (usually 0,pi rad)
  real seg_r1,seg_r2           ! minimum,maximum radial distance of segment from center
  real sx,sz
  integer sxi,szi
  real maxampl
  real scale

  ! integer, parameter :: nxsum=nx1+nx2+nx3+nx4 

  seg_theta1=0.  ! start angle of segment (usually 0 rad)
  seg_theta2= pi ! end angle of segment   (often pi rad)

  scale=(2.**maxlevel)/(2.**maxlevel-1)

  seg_r2=1.
  seg_r1=Dmax/Rmax

  maxampl=max(maxval(abs(src)),maxval(abs(dst_matrix)))*.3

  if(maxampl==0.) then;maxampl=.3;endif
  do px=1,dst_x
    do pz=1,dst_z
      rx=((px-1.)/(dst_x-1.)-.5)*2.         ! % map pixelkoords to intervall [-1.0:1.0]
      rz=((pz-1.)/(dst_z-1.)-.5)*2.
      theta=amod(0.5*pi-atan2(rz,rx),2.*pi) ! kart2pol (rx,rz)-> (theta,radi) */
      radi =sqrt(rx*rx+rz*rz)

      ! check if actual point belongs to current segment and plot it
      if( (theta>=seg_theta1) .and. (theta<=seg_theta2) .and. (radi>=seg_r1) .and. (radi<=seg_r2) ) then
        sx=(theta-seg_theta1)/(seg_theta2-seg_theta1)*(src_x-1)+1;
        sz=(radi-seg_r1)     /(seg_r2-seg_r1)*(src_z-1)+1;
        sxi=sx+.5
        szi=sz+.5

        dst_matrix(px,pz)=src(sxi,src_z-szi+1)
        dst_matrix(dst_x-px+1,pz)=src(sxi,src_z-szi+1)

      ! if we are in the current depth range but outside the current
      ! theta angle, plot background color:
      else if( (theta>seg_theta2) .and. (radi>=seg_r1) .and. (radi<=seg_r2))  then
        dst_matrix(px,pz)        =1*maxampl
        dst_matrix(dst_x-px+1,pz)=1*maxampl
      endif

      if( radi<seg_r1  ) then                  ! pixel px,pz is inside the model
        dst_matrix(px,pz)        =1*maxampl
        dst_matrix(dst_x-px+1,pz)=1*maxampl
      endif

      if( radi>seg_r2 ) then                   ! pixel px,pz is outside the model
        dst_matrix(px,pz)        =1*maxampl
        dst_matrix(dst_x-px+1,pz)=1*maxampl
      endif
    enddo
  enddo
end subroutine map_to_disk

subroutine do_plotting(gif_mode,compo)
  use global
  use plot_mod             ! pgplot module
  implicit none

  integer gif_mode,compo
  integer i,k,x,z
  integer, parameter :: nz_real=nz-8+2  ! assuming that izfree=4 for plotting ->
  integer, parameter :: nx_real=nx-8    ! remove 3 lines from surface and bottom

  ! tmp-arrays are is necessary for correct plotting since a call
  ! like call ANY_FUNCTION(u1(2:nx1-2,2:nz1-2)) leads to an error under linux
  ! remove arrays for large runs (gunnar):
  !
  real tmp1(nx_real,nz_real),tmpval
  integer, parameter :: canvas_x=500,canvas_z=500 ! size of drawing area
  real canvas(canvas_x,canvas_z)                  !  drawing area

  call pgbbuf        ! begin buffer
  call pgpanl(1,1)   ! set panel position
  call pgeras        ! pg erase

  ! set up tmp arrays
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  if(compo==1) then    ! a) plotting of up
    tmp1=0
    do i=4+1,nx-4-1
      do k=4,nz-4
        if(up(i,k)>=0.) then; tmp1(i-4,k-4+1)= (abs(up(i,k)))**gfx_exp
        else;                 tmp1(i-4,k-4+1)=-(abs(up(i,k)))**gfx_exp
        endif
      enddo
    enddo
  endif

  if(compo==3) then    ! a) plotting of mu1
    open(unit=10,file=modelfile)
    tmp1=0
    do x=1,nx
      do z=1,nz
        read(10,*)tmpval
        if( (x<=SIZE(tmp1,DIM=1)) .and. (z<=SIZE(tmp1,DIM=2))) then
        endif
      enddo
    enddo
    close(10)
  endif

  if(compo==2) then    ! a) plotting of mu1
  tmp1=mu1(1:SIZE(tmp1,DIM=1),1:SIZE(tmp1,DIM=2))
  endif

  ! don't plot the surface lines
  canvas(:,:)=0
  call map_to_disk(canvas,canvas_x,canvas_z,tmp1(:,:),nx_real,nz_real) ! map displacement matrix to disk

  call plot_image(canvas,colormap=6)                         ! plot canvas in x-window

  call pgebuf          ! begin buffer
  if(gif_mode==1) then; call pgpage;endif ! for gif,ps,...
end subroutine do_plotting

subroutine do_domain_plotting(gif_mode)
  use global
  use plot_mod             ! pgplot module
  implicit none

  integer gif_mode
  integer i
  ! remove for large runs (gunnar):
  ! real tmp1(nx1,nz1)
  integer, parameter :: canvas_x=500,canvas_z=500 ! size of drawing area
  real canvas(canvas_x,canvas_z)                  !  drawing area

  call pgpanl(1,1)     ! set panel position
  call pgeras          ! erase 
  ! call plot_image(tttmp12,colormap=6)
  call pgpanl(2,1)     ! set panel position
  call pgeras          ! erase 
  ! call plot_image(tttmp22,colormap=6)

   call pgebuf          ! begin buffer
    if(gif_mode==1) then; call pgpage;endif ! for gif,ps,...
end subroutine do_domain_plotting








