!==================================================================================!
!
!  FD_INIT
!
!==================================================================================!
SUBROUTINE fd_init
  USE global 
  USE mod_cart2D
  USE sacoutput
  IMPLICIT NONE
  REAL :: b, evdp, gcarc, omark, stdp     !sacoutput
  REAL :: depth1,depth2,dxmax,dxmin,vmax,dx,aaa,rn
  REAL :: smax,aabs
  REAL :: tmpvmax
  INTEGER :: len                          !sacoutput
  INTEGER :: iz,i,k
  CHARACTER(LEN=80) :: file
  CHARACTER(LEN=8) :: snap_strx! string for rank x-no
  CHARACTER(LEN=8) :: rank_str ! string for file name
  CHARACTER(LEN=90) :: sfile              !actual snapshot file name
  CHARACTER(LEN=8) :: kcmpnm, kstnm       !sacoutput

  write (rank_str,"(I3.3)") mpi_rank
  open(unit=420,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//rank_str(1:3)//'_stdout',status='old',position='append')

  write(420,*)'Begin Init:----:----::----:----::----:----::----:----::----:----::----:----::-:'
  write(420,*)' '

  ! Check number of time steps
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(nt>maxnt) then
    write(420,*)' increase maxnt ... '
    stop
  endif

  ! Determine grid point location for source 
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  do iz=izfree,nz-1
    depth1=rmax-r1(1,iz)
    depth2=rmax-r1(1,iz+1)
    if(sdepth.gt.depth1.and.sdepth.le.depth2) then
      if(sdepth-depth1.le.depth2-sdepth) then
        isz=iz
      else
        isz=iz+1
      endif
    endif
  enddo

  ! write source location error
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  if(mpi_rank==0) then
    write(420,*)'Source originally at: ',sdepth/1000.0, ' [km]'
    write(420,*)'Source mapped to: ',(r1(1,izfree)-r1(1,isz))/1000.0, ' [km]'
    write(420,*)'Source difference: ',abs(sdepth-(r1(1,izfree)-r1(1,isz))), ' [m]'
  endif

  ! initialize fields
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  up=0.
  srp=0.
  stp=0.

  ! angular increment
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  dxmax=rmax*dtheta
  dxmin=(rmax-dmax)*dtheta

  ! time increment
  ! calculate dt from stability
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  vmax=maxval( sqrt(mu1/rho) )
  tmpvmax = vmax  !stores the local vmax
  !determine the maximum vmax of all ranks and store in 'vmax'
  CALL MPI_AllReduce(tmpvmax,vmax,1,MPI_REAL,MPI_MAX,MPI_COMM_WORLD,mpi_ierr)

  dx=min(dxmin,dxmax,dr)
  dt=stab*dx/(sqrt(2.)*vmax)

  nt=nint(stime/dt)

  if(nt>maxnt) then
    write(420,*)nt,' > ',maxnt
    write(420,*)' Increase maxnt and recompile ! '
    stop
  endif

  !Calculate isnap
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  isnap = floor(tsnap/dt)


  ! taper parameters
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  write(420,*) "  "
  write(420,*) "  Initializing taper...."

  aabs=3.*nabs

  window=1.

  ! Gaussian taper function 
  DO i=1,nabs
    taper(nabs+1-i)=EXP(-1./aabs**2*(i-1)**2)  
  ENDDO

  ! initialize 2D damping function window
  
  if(mpi_xrank==dim_size(MPI_X)-1) then ! symmetry at theta=PI
    DO i=1,nabs 
      window(nx+1-i,:)=taper(i) ! define damping for theta_max boundary
    ENDDO                       ! only below 3D Block, HYB: check that !
  endif

  ! 2D normalized damping function 
  window=window/MAXVAL(window)  

  !Uncomment out below lines to write window to file
  !-------------------------------------------------
  !write (snap_strx,"(I2.2)") mpi_rank   ! substring with snapshot no. (00#)
  !file=seisfile(1:len_trim(seisfile))//'_rank'//snap_strx(1:2)//'_win'
  !
  !open(unit=12,file=file,status='unknown')
  !do i=1,nx
  !  do k=1,nz
  !    write(12,*)window(i,k)
  !  enddo
  !enddo
  !close(12)

  ! Source
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  aaa=1./(2.*aa)
  if(aa==0.) then       !disable - not much use for this option
    so(it0)=1.
  else if(aa==-1) then  !Step impulse - best choice for source (seismograms)
    call source_int()
  else                  !Smoothly varying functions for snapshot output
                        ! - best choice for source (snapshots)

    !rn=2.        !d/dt(Gauss) function
    !rn=1.        !Gauss function
    rn=0.001      !step function    <--- This is preferred
    call source(so,nt,dt,aa,rn)

    ! normalize to unity
    smax=0.
    do i=1,nt
      if(abs(so(i)).gt.smax) then
        smax=abs(so(i))
      endif
    enddo
    do i=1,nt
      so(i)=so(i)/smax
    enddo
  endif

  !write source-time function for mpi_rank == 0 (write to a SAC file)
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  IF (mpi_rank == 0) THEN
      IF (sacout == 1) THEN
        len = LEN_TRIM(seisfile)
        sfile = seisfile(1:len)//'_src.sac'
        b = 0.0
        evdp = (rmax-r1(1,isz))/1000.
        stdp =  0.0 
        gcarc = 0.0
        omark = 0.0
        kcmpnm = 'T'
        kstnm = 'source'
        Call wasac(sfile,so,nt,b,dt,evdp,stdp,gcarc,omark,kcmpnm,kstnm,kevnm)
      ENDIF
  ENDIF

  !write some things to '*_inf' file
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  ! new: info file for easy data processing (gunnar, 29.01.2001)
  file=seisfile(1:len_trim(seisfile))//'_inf'  
  open(unit=33,file=file,status='unknown')
  write(33,*)'Simulation_Info'
  write(33,*)'xsize:     nx   = ',nx/iout
  write(33,*)'ysize:     nz   = ',nz/iout
  write(33,*)'timesteps: nt   = ',nt
  write(33,*)'isnap:   isnap  = ',isnap
  write(33,*)'num. of snaps   = ',nt/isnap
  write(33,*)'ranks in x-dir  = ',dim_size(MPI_X) ! added gunnar 6.2002
  write(33,*)'ranks in z-dir  = ',dim_size(MPI_Z) ! for convenient snapshot
  write(33,*)'mpi buffer size = ',BSIZE           ! processing
  write(33,*)'dr (km)         = ',dr/1000.0       ! added mt

  close(33)
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:

  write(420,*)' '
  write(420,*)'End Init:----:----::----:----::----:----::----:----::----:----::----:----::----'
  write(420,*)' '
  close(420)

END SUBROUTINE fd_init
!==================================================================================!


!==================================================================================!
!
!  INITIALISE_R_AND_THETA
!
!==================================================================================!
SUBROUTINE initialise_r_and_theta
  USE global 
  USE mod_cart2D

  character(8) :: snap_strx! string for rank x-no
  character(8) :: snap_strz! string for rank z-no
  integer :: nzr, iz, nxt, ix

  ! initialise r and theta
  !
  ! nzr=nz-2*(izfree-1)                          ! computation for serial version
  nzr=dim_size(MPI_Z)*ZSIZE+2*BSIZE-2*(izfree-1) ! computation for parallel version
  dr=dmax/(nzr-1)

  do iz=1,nz
    r1(:,iz)=rmax-(glob(iz,2)-izfree)*dr
    r2(:,iz)=rmax-(glob(iz,2)-izfree+.5)*dr
  enddo

  ! initialise theta
  !
  ! dtheta=pi/(nxt-1)

  !nxt=nx-2*(izfree-1)                                 ! computation for serial version
  nxt=dim_size(MPI_X)*XSIZE+2*BSIZE-2*(izfree-1)       ! computation for parallel version
  dtheta=tmax/180*pi/(nxt-1)

  !write(*,*)'tmax=',tmax,'pi=',pi,'nxt=',nxt
  do ix=1,nx
    theta1(ix,:)=(glob(ix,1)-izfree)*dtheta
    theta2(ix,:)=(glob(ix,1)-izfree+0.5)*dtheta
  enddo

  !---------------------------------------------------------------------------------------------
  ! Write coordinates to files *_tco and *_rco
  !
  !write (snap_strx,"(I2.2)") mpi_xrank   ! substring with snapshot no. (00#)
  !write (snap_strz,"(I2.2)") mpi_zrank   ! substring with snapshot no. (00#)

  !open(unit=10,file=seisfile(1:LEN_TRIM(seisfile))//'_rx'//snap_strx(1:2)//'rz'//snap_strz(1:2)//'_tco')
  !write(10,*)nx
  !do i=1,nx;   write(10,*)theta1(i,1); enddo
  !close(10)

  !open(unit=11,file=seisfile(1:LEN_TRIM(seisfile))//'_rx'//snap_strx(1:2)//'rz'//snap_strz(1:2)//'_rco')
  !write(11,*)nz
  !do i=1,nz;   write(11,*)r1(1,i); enddo
  !close(11)
  !---------------------------------------------------------------------------------------------

  WHERE(sin(theta1)==0)theta1=1.

  cot1=cos(theta1)/sin(theta1)
  cot2=cos(theta2)/sin(theta2)

  !RETURN theta1 values to correct values for later use of theta1
  !:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:-----:
  IF (mpi_rank==0) THEN
    theta1(izfree,:) = 0.0
  ENDIF 

END SUBROUTINE initialise_r_and_theta
!==================================================================================!


!==================================================================================!
!
!  SOURCE_INT   
!
!  Creates step impulse source time function.
!
!==================================================================================!
SUBROUTINE source_int()
  USE global 
  IMPLICIT NONE
  INTEGER :: i

  so=0.

  do i=it0,maxnt
    so(i)=1.
  enddo

END SUBROUTINE source_int
!==================================================================================!



!==================================================================================!
!
!  SOURCE  
!
!  Used for smooth source time functions (mainly snapshot output)
!
!==================================================================================!
SUBROUTINE source(s,nt,dt,pt,rn)
  IMPLICIT NONE
  real s(*)
  real pt
  real :: pi, t, dt, rn
  integer :: i, nt
  pi=4.*atan(1.)

  do i=1,nt
    t=i*dt
    if(t.lt.pt) then
      s(i) = sin (rn*pi*t/pt) - (rn/(rn+2)) * sin ((rn+2)*pi*t/pt)
    else
      s(i) = sin (rn*pi) - (rn/(rn+2)) * sin ((rn+2)*pi)
    endif
  enddo

END SUBROUTINE source
!==================================================================================!

