!==================================================================================!
!
!  FD_INPUT
!
!==================================================================================!
SUBROUTINE fd_input
  USE global 
  USE mod_cart2D
  IMPLICIT NONE
  CHARACTER(LEN=15) :: junk
  CHARACTER(LEN=50) :: file
  INTEGER :: i
  character(8) :: snap_str ! string for file name

  ! file input
  IF (mpi_rank==0) THEN       ! The root rank (mpi_rank=0) reads

    read(*,*)junk
    read(*,1)junk,seisfile
    read(*,1)junk,modelfile
    read(*,1)junk,recfile
    read(*,3)junk,stime
    read(*,2)junk,nop
    read(*,2)junk,izfree
    read(*,2)junk,igraves
    read(*,2)junk,attenuate
    read(*,2)junk,Nk
    read(*,*)junk
    read(*,2)junk,model_type
    read(*,3)junk,vs0
    read(*,3)junk,rho0
    read(*,3)junk,Q0
    read(*,3)junk,Tmax
    read(*,*)junk
    read(*,2)junk,source_type

    read(*,3)junk,sdepth       !read in source depth in km
      sdepth = sdepth*1000.0   !convert km to meters

    read(*,3)junk,aa
    read(*,2)junk,it0
    read(*,*)junk
    read(*,2)junk,izrec
    read(*,*)junk
    read(*,3)junk,gfx_exp
    read(*,2)junk,icheck
    read(*,3)junk,tsnap
    read(*,2)junk,snapstart
    read(*,2)junk,iout
    read(*,2)junk,iflush
    read(*,2)junk,sacout
    read(*,1)junk,kevnm
    read(*,2)junk,nabs

  ENDIF ! endif mpi_rank

 1	format(a,a)
 2	format(a,i8)
 3	format(a,f10.6)

  ! Now the "<" redirection works properly for >=1 Process.
  !
  ! Note: When the input file format changes, modifications must be applied
  ! both at the read()-statement section *and* the MPI_Broadcast section.
  !
  call MPI_BCAST( seisfile   ,50, MPI_CHARACTER, 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( modelfile  ,50, MPI_CHARACTER, 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( recfile    ,50, MPI_CHARACTER, 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( stime      , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( nop        , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( izfree     , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( igraves    , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( attenuate  , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( Nk         , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( model_type , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( vs0        , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( rho0       , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( Q0         , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( Tmax       , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( source_type, 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( sdepth     , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( aa         , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( it0        , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( izrec      , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( gfx_exp    , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( icheck     , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( tsnap      , 1, MPI_REAL     , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( snapstart  , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( iout       , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( iflush     , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( sacout     , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( kevnm      ,16, MPI_CHARACTER, 0, MPI_COMM_WORLD, mpi_ierr )
  call MPI_BCAST( nabs       , 1, MPI_INTEGER  , 0, MPI_COMM_WORLD, mpi_ierr )


  !Initialise r, theta
  !--------------------------------------------------
  call initialise_r_and_theta


  !Read in receiver locations
  !--------------------------------------------------
  open(unit=10,file=recfile)
  read(10,*)nr
  do i=1,nr
  read(10,*)rec_loc(i)
  enddo
  close(10)


  !Write out parameters read in, to '*_input' files
  !--------------------------------------------------
  write (snap_str,"(I3.3)") mpi_rank
  open(unit=10,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//snap_str(1:3)//'_input',status='unknown')
    write(10,*)'seisfile=',seisfile
    write(10,*)'modelfile=',modelfile
    write(10,*)'recfile=',recfile
    write(10,*)'stime=',stime
    write(10,*)'nop=',nop
    write(10,*)'izfree=',izfree
    write(10,*)'igraves=',igraves
    write(10,*)'attenuate=',attenuate
    write(10,*)'Nk=',Nk
    write(10,*)'model_type=',model_type
    write(10,*)'vs0=',vs0
    write(10,*)'rho0=',rho0
    write(10,*)'Q0=',Q0
    write(10,*)'Tmax=',Tmax
    write(10,*)'source_type=',source_type
    write(10,*)'sdepth=',sdepth
    write(10,*)'aa=',aa
    write(10,*)'it0=',it0
    write(10,*)'izrec=',izrec
    write(10,*)'gfx_exp=',gfx_exp
    write(10,*)'icheck=',icheck
    write(10,*)'tsnap=',tsnap
    write(10,*)'snapstart=',snapstart
    write(10,*)'iout=',iout
    write(10,*)'iflush=',iflush
    write(10,*)'sacout=',sacout
    write(10,*)'kevnm=',kevnm
    close(10)

END SUBROUTINE fd_input
!==================================================================================!

