!==================================================================================!
!
!  *     *     *     *     *     *     *     *     *     *     *     *     *     * 
!
!                                        S H A X I
!
!  *     *     *     *     *     *     *     *     *     *     *     *     *     * 
!
!  FD2_MAIN
!
!  SH wave propagation in spherical coordinates
!
!  By, Gunnar Jahnke, Michael Thorne, and Heiner Igel
!
!  Reference:  Jahnke, Thorne, Cochard, and Igel (2007),
!              Global SH-wave propagation using a parallel axi-symmetric
!              finite-difference scheme, Geophysical Journal International, 
!              Vol. (??), doi:(??), pp.
!
!==================================================================================!
PROGRAM fd2_main
  USE global
  USE mod_cart2D
  USE attenuation
  IMPLICIT NONE 
  INTEGER :: nprocs

  !Initialize MPI environment
  CALL MPI_Init(mpi_ierr)   

  !Determine number of requested processes
  CALL MPI_Comm_size(MPI_COMM_WORLD, nprocs, mpi_ierr)
  dim_size(MPI_X)=nprocs  !Number of processes along horizontal axis
  dim_size(MPI_Z)=1       !Number of processes along vertical axis

  
  !Check number of processors
  CALL my_mpi_check( dim_size(MPI_X)*dim_size(MPI_Z) ) 
  !Initialize Cartesian topology
  CALL my_cart_init()    
  !Allocate space variables
  CALL fd_allocate        

  srp=0;stp=0;up=0

  !Attenuation
  Q1=0.0; Q2=0.0

  CALL fd_input  !Read input parameters from file
  CALL fd_model  !Read model
  CALL fd_init   !Initialize Setup

  !Calculate isnap
  !isnap = floor((nt*dt)/tsnap)

  !Attenuation
  IF (attenuate == 1) THEN
    CALL Q_init
  ENDIF

  CALL fd_check  !Check Setup

  !Time evolution
  DO it=1,nt
    CALL fd_evolution        
    CALL fd_output                  
  ENDDO 

  CALL MPI_Finalize(mpi_ierr) 

END PROGRAM fd2_main
!==================================================================================!

