!==================================================================================!
!
!  FD_OUTPUT
!
!==================================================================================!	
SUBROUTINE fd_output
  USE global
  USE mod_cart2D
  IMPLICIT NONE
  REAL :: t2
  REAL :: vtheta(nx),utmp(nx),u2tmp(nx,nz)
  INTEGER :: i,iiout,iss,j
  CHARACTER(LEN=8)  :: rank_str ! string for file name
  CHARACTER(LEN=62) :: timestr
  CHARACTER(LEN=50) :: file

  write (rank_str,"(I3.3)") mpi_rank


  !Write out some timeing information
  !------------------------------------------------------------------------------------
  IF (it == 1) THEN
    CALL timer(syst1)
  ENDIF

  IF (mod(it,icheck) == 0) THEN
    open(unit=420,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//rank_str(1:3)//'_stdout',status='old',position='append')
    CALL timer(t2) 
    CALL tstr(syst1,t2,timestr)
    write (420,*) "Run Time At it:",it," == ", TRIM(adjustl(timestr)), " ..."
    t2 = syst2 +((t2-syst2)/(icheck))*nt
    CALL tstr(syst2,t2,timestr)
    write (420,*) "Estimated Total Run Time At it:",nt," == ", TRIM(adjustl(timestr)), " ..."
    CALL timer(syst2)
    close(420)
  ENDIF

  ! gathering seismograms during simulation
  !------------------------------------------------------------------------------------
  vtheta=theta1(:,1)
  utmp=up(:,izrec)

  do i=1,nr
    if( my_position(rec_loc(i)/180.*pi,1)==1) then
      iiout=(i-1)*nt+it
      call rec_int(vtheta,nx,rec_loc(i)/180.*pi, utmp,seis(iiout))
    endif
  enddo

  !write out seismograms
  if(mod(it,iflush)==0 .or.it==nt) then
    call seis_output()  
  endif

  ! snapshot output
  !------------------------------------------------------------------------------------
  if(isnap>0) then
    if(mod(it,isnap)==0 .and. (it/isnap) >= snapstart) then
      iss=it/isnap
      if(mpi_rank==0) then
        open(unit=420,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//rank_str(1:3)//'_stdout',status='old',position='append')
        write(420,*)' Writing snapshot ',iss,' ... '
        close(420)
      endif
      u2tmp=up
      call snap_output(u2tmp,it,nx,nz,isnap)
    endif
  endif

END SUBROUTINE fd_output
!==================================================================================!	


!==================================================================================!
!
!  REC_INT
!
!==================================================================================!	
subroutine  rec_int(x,nx,xr,f,out)
  ! this routine interpolates linearly a scalar field known at 
  ! x(0:nx) and y(0:ny) locations to an arbitrary location
  ! within xmax and ymax.
  ! The receiver locations are xr, and yr, the scalar input
  ! field is plane and the function value at xr and yr is
  ! out.

  integer :: nx, i, ilow, iup
  real x(nx)
  real xr,yr,out
  real f(nx)
  real :: dx, hx, f1, f2

  do i=0,nx
    if(xr .ge. x(i) .and. xr .lt. x(i+1) ) then
      ilow=i
      iup =i+1
      dx=x(i+1)-x(i)
      hx=xr-x(i)
    endif
  enddo

  f1=f(ilow)
  f2=f(iup)

  out = f1 + (f2-f1)/dx*hx
  return
end
!==================================================================================!	


!==================================================================================!
!
!  SNAP_OUTPUT
!
!==================================================================================!	
! subroutine snap_output
! taken from "/acoustic_sndrcv/source/main.f90"
!
! Writes the actual field stored in 'p' to file.
!
! The filename created by the subroutine contains
!    * the rank position in x- and y-direction and
!    *  the current snapshot number.
!
! Input parameters:
!   p     : field of size (nx,nz) to use for snapshot
!   it    : actual time step
!   nx,nz : size of the array to snapshot            ( now in module!)
!   isnap : create snapshot every 'isnap' time steps
!
subroutine snap_output(p,it,nx,nz,isnap)
!  use global, only : seisfile,iout
  use global, only : seisfile, iout, dt, r1, theta1, snapstart !*mt*
  use mod_cart2D
  implicit none

  integer :: it,nx,nz,isnap,len
  real p(nx,nz)              ! data for snapshot file 

  character*90 sfile       ! actual snapshot file name
  character(8) :: snap_str ! string for snapshot name
  character(8) :: snap_strx! string for rank x-no
  character(8) :: snap_strz! string for rank z-no

  character*50 file          ! file name (definition necessary?)

  integer ix,iy              ! indices

  write (snap_strx,"(I2.2)") mpi_xrank   ! substring with snapshot no. (00#)
  write (snap_strz,"(I2.2)") mpi_zrank   ! substring with snapshot no. (00#)
  len=LEN_TRIM(seisfile)

  if(isnap/=0) then                         ! no snapshots for isnap==0
    if(mod(it,isnap)==0 .and. it >= snapstart) then ! time for another snapshot
      write (snap_str,"(I3.3)") it/isnap   ! substring with snapshot no. (00#)

      ! merge the whole snapshot file name
      sfile = seisfile(1:len)//'_rx'//snap_strx(1:2)//'rz'//snap_strz(1:2)//'snap'//snap_str(1:3)
      open(unit=12,file=sfile,status='unknown')
      write(12,*) iout                !*mt* samples
      write(12,*) dt                  !*mt* time increment
      write(12,*) it                  !*mt* output time
      write(12,*) nz                  !*mt* radial number of grid points
      write(12,*) minval(r1)          !*mt* minimum radius
      write(12,*) maxval(r1)          !*mt* maximum radius
      write(12,*) nx                  !*mt* theta grid points
      write(12,*) minval(theta1)      !*mt* minium theta
      write(12,*) maxval(theta1)      !*mt* maximum theta
      do ix=1,nx,iout
        do iy=1,nz,iout
          write(12,*)p(ix,iy)
        enddo
      enddo
      close(12)
    endif
  endif
  return
end
!==================================================================================!	


!==================================================================================!
!
!  SEIS_OUTPUT
!
!==================================================================================!	
subroutine seis_output()
  use global
  use mod_cart2D
  use sacoutput                           !sacoutput
  implicit none

  character(len=90) :: sfile              !actual snapshot file name
  character(8) :: snap_str                !strings for snapshot name
  CHARACTER(len=8) :: kcmpnm, kstnm       !sacoutput
  REAL, DIMENSION(nt) :: yarray           !sacoutput
  REAL :: b, evdp, gcarc, omark, stdp     !sacoutput
  integer len,i,k
  character(8) :: rank_str ! string for file name
  write (rank_str,"(I3.3)") mpi_rank

  len=LEN_TRIM(seisfile)

  if(mpi_rank==0) then
    open(unit=420,file=seisfile(1:LEN_TRIM(seisfile))//'_rank'//rank_str(1:3)//'_stdout',status='old',position='append')
    write(420,*)'writing seismograms'
    close(420)
  endif

  do i=1,nr

    if( my_position(rec_loc(i)/180.*pi,1)==1) then
      if (sacout == 0) then
        write (snap_str,"(I3.3)") i   ! substring with seismometer-no
        sfile = seisfile(1:len)//'_seis'//snap_str(1:3)
        open(unit=20,file=sfile,status='unknown')
        write(20,*) nt                      !Total time steps 
        write(20,*) dt                      !Time spacing
        write(20,*) (rmax-r1(1,isz))/1000.  !Actual Source Depth (km)
        write(20,*) rec_loc(i)              !Receiver Location (deg)
        write(20,*) it0                     !Delta-Function time
        write(20,*) snap_str(1:3)           !Receiver Number
        write(20,*) nop                     !Operator Length
        write(20,*) nx                      !x-grid
        write(20,*) nz                      !z-grid
        write(20,*) dim_size(MPI_X)         !X ranks
        write(20,*) model_type              !model type 
        do k=1,nt
          write(20,*)seis((i-1)*nt+k)
        enddo
        close(20)
      endif

      if (sacout == 1) then
        write (snap_str,"(I3.3)") i   ! substring with seismometer-no
        sfile = seisfile(1:len)//'_'//snap_str(1:3)//'.T'
        b = 0.0
        evdp = (rmax-r1(1,isz))/1000.
        stdp = (rmax-r1(1,izrec))/1000.
        gcarc = rec_loc(i)
        omark = dt*float(it0)
        kcmpnm = 'T'
        kstnm = snap_str(1:3) 
        do k=1,nt
          yarray(k) = seis((i-1)*nt+k)
        enddo
        Call wasac(sfile,yarray,nt,b,dt,evdp,stdp,gcarc,omark,kcmpnm,kstnm,kevnm) 
      endif

    endif ! endif my_position
  enddo ! enddo i
end
!==================================================================================!


!==================================================================================!
!
!  TSTR
!
!==================================================================================!	
SUBROUTINE tstr(t1,t2,timestr)
IMPLICIT NONE
REAL, INTENT(IN) :: t1, t2
REAL :: dif
INTEGER :: day, hour, minn, sec
CHARACTER(LEN=62), INTENT(OUT) :: timestr
CHARACTER(LEN=8) :: dstr, mstr, hstr, sstr

dif = t2 - t1

day = dif/86400.0
hour = (dif - 86400*float(day))/3600.0
minn = (dif - (86400*float(day) + 3600.0*float(hour)))/60.0
sec = dif - (86400*float(day) + 3600.0*float(hour) + 60.0*float(minn))

write(dstr,'(I4)') day
write(hstr,'(I4)') hour
write(mstr,'(I4)') minn
write(sstr,'(I4)') sec

timestr = TRIM(adjustl(dstr))//' days '//TRIM(adjustl(hstr))//' hours '//TRIM(adjustl(mstr)) &
& //' minutes '//TRIM(adjustl(sstr))//' seconds'

END SUBROUTINE tstr
!==================================================================================!



!==================================================================================!
!
!  TIMER
!
!==================================================================================!	
SUBROUTINE timer(csec)
IMPLICIT NONE
REAL, INTENT(OUT) :: csec
INTEGER, DIMENSION(8) :: v
INTEGER, DIMENSION(24) :: itndpmo
INTEGER :: im, id, iyyy
INTEGER :: julday
INTEGER :: I
CHARACTER(8) :: date
CHARACTER(10) :: time
CHARACTER(5) :: zone

CALL DATE_AND_TIME(date,time,zone,v)

iyyy = v(1)
im = v(2)
id = v(3)

itndpmo(1:12)=(/0,31,59,90,120,151,181,212,243,273,304,334/)
itndpmo(13:24)=(/0,31,60,91,121,152,182,213,244,274,305,335/)
IF (MOD(iyyy,4) == 0) THEN
  IF (MOD(iyyy,100) == 0 .and. MOD(iyyy,400) /= 0) THEN
    I = im
  ELSE
    I = im + 12
  ENDIF
ELSE
I = im
ENDIF
julday=itndpmo(I) + id
csec = float(julday)*86400.0 + float(v(5))*3600.0 + float(v(6))*60.0 + float(v(7))

END SUBROUTINE timer
!==================================================================================!	


