!==================================================================================!
!
!  MODULE:   GLOBAL
!
!  global specifications
!
!  module contains all global variables
!
!==================================================================================!
MODULE global

  IMPLICIT NONE 
  !*     *     *     *     *     *     *    *     *     *     *     *     *    *   !
  !================================================================================!
  !
  !   THESE ARE THE MAIN VARIABLES THAT MAY NEED ALTERATION BEFORE COMPILING
  !
  !================================================================================!

  ! nx,ny: grid size in horiz/vert direction
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  ! SOME SUGGESTIONS                                  !S(80 deg)  (2700 sec sim)
  !integer, parameter :: XSIZE=5000/4,   ZSIZE=1000   !Tdom=18s   Time=    21m
  integer, parameter :: XSIZE=10000/4,  ZSIZE=1800   !Tdom=12s   Time= 2h 16m
  !integer, parameter :: XSIZE=10000/8,  ZSIZE=1800   !Tdom=12s   Time= 1h 24m 
  !integer, parameter :: XSIZE=15000/8,  ZSIZE=2900   !Tdom=10s   Time= 4h 24m
  !integer, parameter :: XSIZE=15000/12, ZSIZE=2900   !Tdom=10s   Time= 3h 32m
  !integer, parameter :: XSIZE=20000/12, ZSIZE=3800   !Tdom= 8s   Time= 8h 14m
  !integer, parameter :: XSIZE=20000/16, ZSIZE=3800   !Tdom= 8s   Time= 6h 23m
  !integer, parameter :: XSIZE=30000/16, ZSIZE=5200   !Tdom= 6s   Time=17h 38m
 
  ! Size of model
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  REAL, PARAMETER :: Rmax=6371000. ! the earth's radius
  REAL, PARAMETER :: Dmax=2891000. ! depth of the CMB == thickness model in z-direction

  !  Number of time steps and receivers
  !~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  INTEGER, PARAMETER :: maxnt=150000   !Maximum number of time steps
  INTEGER, PARAMETER :: maxnr=200      !Maximum number of receivers

  !================================================================================!
  !*     *     *     *     *     *     *    *     *     *     *     *     *    *   !
  !
  !                    Nothing below here should be changed
  !
  !*     *     *     *     *     *     *    *     *     *     *     *     *    *   !




  !===========================================================================!
  !
  ! Parameters
  !
  !===========================================================================!
  INTEGER, PARAMETER :: BSIZE=4             !Buffer Size
  INTEGER, PARAMETER :: nx=XSIZE+2*BSIZE    !Number of lateral gridpoints (per
                                            !   rank)
  INTEGER, PARAMETER :: nz=ZSIZE+2*BSIZE    !Number of radial gridpoints
  REAL,    PARAMETER :: pi=3.1415926535898
  REAL,    PARAMETER :: stab = 0.75         !Stability criterion
  !===========================================================================!


  !===========================================================================!
  !
  ! SHaxi Allocatable Arrays
  !
  !===========================================================================!
  REAL, DIMENSION(:,:), ALLOCATABLE :: srp      !Stress (r,phi)
  REAL, DIMENSION(:,:), ALLOCATABLE :: stp      !Stress (theta,phi)
  REAL, DIMENSION(:,:), ALLOCATABLE :: up       !Velocity
  REAL, DIMENSION(:,:), ALLOCATABLE :: rho      !Density 
  REAL, DIMENSION(:,:), ALLOCATABLE :: mu1      !Shear modulus (grid 1)
  REAL, DIMENSION(:,:), ALLOCATABLE :: mu2      !Shear modulus (grid 2)
  REAL, DIMENSION(:,:), ALLOCATABLE :: theta1   !Theta grid locations (grid 1)
  REAL, DIMENSION(:,:), ALLOCATABLE :: theta2   !Theta grid locations (grid 2)
  REAL, DIMENSION(:,:), ALLOCATABLE :: cot1     !Cotangent (grid 1)
  REAL, DIMENSION(:,:), ALLOCATABLE :: cot2     !Cotangent (grid 2)
  REAL, DIMENSION(:,:), ALLOCATABLE :: r1       !Radial grid locations (grid 1)
  REAL, DIMENSION(:,:), ALLOCATABLE :: r2       !Radial grid locations (grid 2)
  !===========================================================================!


  !===========================================================================!
  !
  ! SHaxi Static Arrays
  !
  !===========================================================================!
  REAL, DIMENSION(nx,nz)       :: window       !Damping window  
  REAL, DIMENSION(MAX(nx,nz))  :: taper        !Damping taper function
  REAL, DIMENSION(maxnt)       :: so           !Vector for source-time f'n
  REAL, DIMENSION(maxnr)       :: rec_loc      !Receiver locations 
                                               !  angle theta (deg)
  REAL, DIMENSION(maxnr*maxnt) :: seis         !Vector for seismograms
                                               !  order is: seis(:)=
                                               !  (seis1(1:nt),seis2(1:nt),...)
  !===========================================================================!


  !===========================================================================!
  !
  ! SHaxi Real Variables
  !
  !===========================================================================!
  REAL :: Tmax        ! angular size of model, 180 DEG= full earth 
  REAL :: rho0        ! density for homogeneous model
  REAL :: vs0         ! velocity for homogeneous model
  REAL :: tsnap       ! time interval between snapshots (in seconds)
  REAL :: syst1       ! time it=1 (system time)
  REAL :: syst2       ! time (system time during runtime)
  REAL :: aa          ! dominant period of source signal
  REAL :: dt          ! time increment
  REAL :: dtheta      ! angular grid spacing (increment of angle theta)
  REAL :: dr          ! radial grid spacing
  REAL :: sdepth      ! source depth [m]
  REAL :: stime       ! simulation time (in seconds)
  REAL :: gfx_exp     ! exponent for snapshot graphics, u^exp will be displayed
  !===========================================================================!

  !===========================================================================!
  !
  ! SHaxi Integer Variables
  !
  !===========================================================================!
  INTEGER :: source_type ! source type =  only source_type == 1 is enabled.
                         !       1 : upi=upi+so   
  INTEGER :: isz         ! z-index of source position, not used
  INTEGER :: nabs        ! number of grid points to use for absorbing boundary
  INTEGER :: model_type  ! model type, (1)=>homogeneous,rho=rho0,mu1=mu2=rho*vs0^2
                         ! (2)=>prem ; ... other's user models 
  INTEGER :: icheck      ! every 'icheck' time step do output of predicted runtime 
  INTEGER :: nt          ! number of time steps for simulation
  INTEGER :: it0         ! start time of source signal 
  INTEGER :: it          ! actual iteration
  INTEGER :: izfree      ! position of free surface / x-position of source, depends on 'nop'
  INTEGER :: igraves     ! flag for surface method, 1=graves 0=symmetry
  INTEGER :: nop         ! operator length, nop=2/4/8
  INTEGER :: izrec       ! receiver depth in grid points
  INTEGER :: iout        ! snapshot : output every 'iout' grid point
  INTEGER :: isnap       ! do snapshot output every 'isnap' time steps
  INTEGER :: snapstart   ! begin writing snapshots with 'snapstart' snapshot
  INTEGER :: nr          ! number of receivers
  INTEGER :: iflush      ! do seismogram output every 'iflush' time steps
  INTEGER :: sacout      ! sacoutput 0=no (ascii) 1=yes alphanumeric SAC output
  !===========================================================================!


  !===========================================================================!
  !
  !Variables for attenuation
  !
  !===========================================================================!
  INTEGER :: attenuate                   !Attenuation Switch (0=off, 1=on)
  INTEGER :: Nk                          !Number of memory variables (1-8) 

  REAL :: Q0                                           !Quality factor for 
                                                       !homogeneous media

  REAL, DIMENSION(8) :: rlxt                           !Relaxation times

  REAL, DIMENSION(:,:),   ALLOCATABLE :: Q1, Q2        !Quality factors
  REAL, DIMENSION(:,:),   ALLOCATABLE :: Mu_1, Mu_2    !Unrelaxed Modulus
  REAL, DIMENSION(:,:,:), ALLOCATABLE :: mvr1, mvr2    !Memory Variables
  REAL, DIMENSION(:,:,:), ALLOCATABLE :: wtr1, wtr2    !Weight Functions

  !===========================================================================!


  !===========================================================================!
  !
  ! Files
  !
  !===========================================================================!
  CHARACTER(len=50) :: seisfile   ! prefix for output files
  CHARACTER(len=50) :: recfile    ! input file with receiver coordinates
  CHARACTER(len=50) :: modelfile  ! for model_type=3 
  CHARACTER(len=16) :: kevnm      ! if sacout==1 16,character description of run
  !===========================================================================!


END MODULE global
!==================================================================================!

