
module plot_mod

  implicit none

  character(len=8), public :: plot_time, plot_step  
  character(len=30), public :: plot_xlab, plot_ylab, plot_zlab, plot_title, plot_device
  integer, public :: plot_ipan, plot_jpan
  real, public :: plot_ratio
  !------------------------------------------------------------------------------------
!  interface
!     subroutine plot_parameter(parameterfile)
!       implicit none
!       character(len=30) :: parameterfile
!     end subroutine plot_parameter
!  end interface
  !__________________________________________________________________
  interface
     subroutine plot_dots(y,x,xlab,ylab,title,symbol)
       real, dimension(:), intent(in) :: Y
       real, dimension(:), intent(in),optional :: X
       character(len=30),intent(in), optional :: xlab, ylab, title
       integer, intent(in), optional ::  symbol
     end subroutine plot_dots
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_2dots(x1,y1,x2,y2,xlab,ylab,title,symbol1,symbol2,color1,color2)
       integer, intent(in), optional ::  symbol1,symbol2,color1,color2
       real, dimension(:), intent(in) :: Y1,y2
       real, dimension(:), intent(in) :: X1,x2
       character(len=30), optional :: xlab, ylab, title
     end subroutine plot_2dots
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_line(y,x,xlab,ylab,title)
       real, dimension(:), intent(in) :: Y
       real, dimension(:), intent(in), optional :: X
       character(len=30),intent(in), optional :: xlab, ylab, title
     end subroutine plot_line
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_2lines(x1,y1,x2,y2,xlab,ylab,title,color1,color2)
       real, dimension(:), intent(in) :: Y1,y2
       real, dimension(:), intent(in) :: X1,x2
       character(len=30), intent(in), optional :: xlab, ylab, title
       integer, intent(in), optional :: color1,color2
     end subroutine plot_2lines
  end interface
  !__________________________________________________________________ plot_ipan=3
  interface
     subroutine plot_3lines(x1,y1,x2,y2,x3,y3,xlab,ylab,title,color1,color2,color3)
       real, dimension(:), intent(in) :: Y1,y2,y3
       real, dimension(:), intent(in) :: X1,x2,x3
       character(len=30), intent(in),optional :: xlab, ylab, title
       integer, intent(in), optional :: color1,color2,color3
     end subroutine plot_3lines
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_multiline(x,y,xlab,ylab,title)
       real, dimension(:,:) :: Y
       real, dimension(:) :: X
       character(len=30), optional :: xlab, ylab, title
     end subroutine plot_multiline
  end interface
  !__________________________________________________________________
  interface 
     subroutine plot_init(device,xwinsize,ratio,ipan,jpan,font)
      character(len=30), intent(in) :: device 
       real, intent(in), optional :: xwinsize, ratio
       integer, intent(in), optional :: ipan, jpan, font
     end subroutine plot_init
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_image(field,xlab,ylab,zlab,title,colormap,fieldmax,box) 
       real, dimension(:,:), intent(in) :: field
       character(len=30), optional :: xlab, ylab, zlab, title
       integer, intent(in), optional :: colormap, box
       real, intent(in), optional :: fieldmax
     END subroutine plot_image
  end interface
  !__________________________________________________________________
  interface
     subroutine plot_vectfield(field1,field2,i1,i2,j1,j2,xlab,ylab,zlab,title,norm)
       real, dimension(:,:), intent(in) :: field1,field2
       integer, intent(in) :: i1,i2,j1,j2
       character(len=30), optional :: xlab, ylab, zlab, title
       integer, intent(in), optional :: norm 
     END subroutine plot_vectfield
  end interface
  !__________________________________________________________________
  interface
     SUBROUTINE PALETT(TYPE, CONTRA, BRIGHT)
       INTEGER TYPE
       REAL CONTRA, BRIGHT
     END subroutine PALETT
  end interface
  !__________________________________________________________________
  interface
     SUBROUTINE OUTLIN(I1,I2,J1,J2,TR)
       INTEGER I1,I2,J1,J2
       REAL TR(6)
     end subroutine OUTLIN
  end interface
  !__________________________________________________________________
!  interface
!     function pglnblnk(c)
!       implicit none  	
!       character(len=*):: c
!     end function pglnblnk
!  end interface
  !__________________________________________________________________
end module plot_mod

!##################################################################################
! module subroutine codes:
!
!  subroutine plot_parameter(parameterfile)!

!    use plot_mod

!    implicit none
!    integer :: i,n
!    real :: x, y, X1, X2, Y1, Y2
!    character(len=100):: parastring
!    character(len=*) :: parameterfile

!    open(8, file=parameterfile)
!    do n = 1, 1000
!       read(8,'(a100)',end=111) parastring
!       write(*,*)parastring
!    enddo
!111 continue
!    write(*,*) 'number of parameters : ', n
!    close(8)
!    open(9, file=parameterfile(1:pglnblnk(parameterfile)))
!    call PGSCF(4)
!    CALL PGSCH(.9)
!    call pgbbuf
!    call pgpanl(1,1)
!    call pgeras
!    call PGQVP (0, X1, X2, Y1, Y2)
!    write(*,*) 'viewport coordinates',x1,x2,y1,y2
!    call PGQVSZ(0, X1, X2, Y1, Y2)
!    write(*,*) 'viewsurface coordinates',x1,x2,y1,y2
!    call PGQWIN (X1, X2, Y1, Y2)
!    write(*,*) 'window boundary coordinates',x1,x2,y1,y2
!    do i = 1, n-1
!       X = 0.0
!       Y = (1.-REAL(i)/real(n))*(y2-y1)
!       read(9,'(a100)') parastring
!       parastring = adjustl(trim(parastring)) 
!       write(*,*) X,Y,  parastring
!     call pgtext(X, Y, parastring)
!    enddo
!    call pgebuf
!    close(9)
!  end subroutine !plot_parameter
!__________________________________________________________________

  subroutine plot_dots(y,x,xlab,ylab,title,symbol)
    ! dummy arguments
    real, dimension(:), intent(in) :: Y
    real, dimension(:), intent(in),optional :: X
    character(len=30),intent(in), optional :: xlab, ylab, title
    integer, intent(in), optional ::  symbol
    !local variables
    real, dimension(:), allocatable :: xtmp
    character(len=30) :: xla, yla, tit
    integer :: i, sym

    
    CALL PGVSTD
    select case(present(symbol));case(.false.);sym=1;case(.true.);sym=symbol;end select	
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select    
    call pglab(xla,yla,tit)
    if (present(x)) then 
      CALL PGSWIN (minval(x),maxval(x),minval(y),maxval(y))
      CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
      call pgpt(size(y),x,y,sym)
    else
      write(*,*)'hallo1'
      allocate(xtmp(1:size(y)))
      do i = 1, size(y)
         xtmp(i) = real(i)      	
      enddo
      CALL PGSWIN (minval(xtmp),maxval(xtmp),minval(y),maxval(y))
      CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
      call pgpt(size(y),xtmp,y,sym)
       deallocate(xtmp)
    endif

  end subroutine plot_dots
!__________________________________________________________________
  subroutine plot_2dots(x1,y1,x2,y2,xlab,ylab,title,symbol1,symbol2,color1,color2)
    ! dummy arguments
    integer, intent(in), optional ::  symbol1,symbol2,color1,color2
    real, dimension(:), intent(in) :: Y1,y2
    real, dimension(:), intent(in) :: X1,x2
    character(len=30), optional, intent(in) :: xlab, ylab, title
    ! local variables
    character(len=30) :: xla, yla, tit
    real :: minx,maxx,miny,maxy
    integer :: color
!    CALL PGPAGE
    minx=minval(min(x1,x2))
    miny=minval(min(y1,y2))
    maxx=maxval(max(x1,x2))
    maxy=maxval(max(y1,y2))

    CALL PGVSTD
    CALL PGSWIN (minx,maxx,miny,maxy)
    CALL PGBOX ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
    call pgqci(color)
    call pgsci(color1)
    call pgpt(size(y1),x1,y1,symbol1)
    call pgsci(color2)
    call pgpt(size(y2),x2,y2,symbol2)
    call pgsci(color)
  end subroutine plot_2dots
!__________________________________________________________________

  subroutine plot_line(y,x,xlab,ylab,title)
    ! dummy arguments
    real, dimension(:),intent(in) :: y
    real, dimension(:), intent(in), optional :: x
    character(len=30),intent(in), optional :: xlab, ylab, title
    ! internal arguments
    real, dimension(:), allocatable :: xtmp
    character(len=30) :: xla, yla, tit
    integer :: i

    CALL PGVSTD
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select    
    call pglab(xla,yla,tit)
    if (present(x)) then 
      CALL PGSWIN (minval(x),maxval(x),minval(y),maxval(y))
      CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
      call pgline(size(y),x,y)
    else
      write(*,*)'hallo1'
      allocate(xtmp(1:size(y)))
      do i = 1, size(y)
         xtmp(i) = real(i)      	
      enddo
    CALL PGSWIN (minval(xtmp),maxval(xtmp),minval(y),maxval(y))
    CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    call pgline(size(y),xtmp,y)
    deallocate(xtmp)
    endif

 end subroutine plot_line
!__________________________________________________________________

  subroutine plot_2lines(x1,y1,x2,y2,xlab,ylab,title,color1,color2)
    ! dummy arguments
    real, dimension(:), intent(in) :: Y1,y2
    real, dimension(:), intent(in) :: X1,x2
    character(len=30), optional, intent(in) :: xlab, ylab, title
    integer, intent(in), optional ::color1, color2
    ! local variables
    character(len=30) :: xla, yla, tit
    real :: minx,maxx,miny,maxy
    integer:: color

    minx=minval(min(x1,x2))
    miny=minval(min(y1,y2))
    maxx=maxval(max(x1,x2))
    maxy=maxval(max(y1,y2))
    CALL PGVSTD
    CALL PGSWIN (minx,maxx,miny,maxy)
    CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
    call pgqci(color)
    if (present(color1))then
       call pgsci(color1)
    else
       call pgsci(1)
    endif
    call pgline(size(y1),x1,y1)
    if (present(color2))then
       call pgsci(color2)
    else
       call pgsci(2)
    endif    
    call pgline(size(y2),x2,y2)
    call pgsci(color)

  end subroutine plot_2lines
!__________________________________________________________________

  subroutine plot_3lines(x1,y1,x2,y2,x3,y3,xlab,ylab,title,color1,color2,color3)
    ! dummy arguments
    real, dimension(:), intent(in) :: Y1,y2,y3
    real, dimension(:), intent(in) :: X1,x2,x3
    character(len=30), intent(in), optional :: xlab, ylab, title
    integer, intent(in), optional :: color1, color2, color3
    ! local variables
    character(len=30):: xla, yla, tit
    real :: minx,maxx,miny,maxy
    integer:: color

    minx=minval(min(x1,x2,x3))
    miny=minval(min(y1,y2,y3))
    maxx=maxval(max(x1,x2,x3))
    maxy=maxval(max(y1,y2,y3))
    CALL PGVSTD
    CALL PGSWIN (minx,maxx,miny,maxy)
    CALL PGBOX  ('BCNST', 0.0, 0, 'BCNST', 0.0, 0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
    call pgqci(color)
    if (present(color1))then
       call pgsci(color1)
    else
       call pgsci(1)
    endif
    call pgline(size(y1),x1,y1)
    if (present(color2))then
       call pgsci(color2)
    else
       call pgsci(2)
    endif
    call pgline(size(y2),x2,y2)
    if (present(color3))then
       call pgsci(color3)
    else
       call pgsci(3)
    endif
    call pgline(size(y3),x3,y3)
    call pgsci(color)

  end subroutine plot_3lines
!__________________________________________________________________
  subroutine plot_multiline(x,y,xlab,ylab,title)

    integer :: I, pgopen, symbol
    real, dimension(:,:) :: Y
    real, dimension(:) :: X
    character(len=30),intent(in), optional :: xlab, ylab, title
    ! local variables 
    character(len=30):: xla, yla, tit

!    call pgunsa
!    call pgvstd
    call pgsvp(0.15,0.85,0.15,0.85)
    call pgswin(MINVAL(X)-(Maxval(X)-MINVAL(X))/50.,Maxval(X)+(Maxval(X)-MINVAL(X))/50. & 
              &,MINVAL(Y)-(MAXVAL(Y)-MINVAL(Y))/5.,MAXVAL(Y)+(MAXVAL(Y)-MINVAL(Y))/5.) 
!    call pgsave
    call PGSCI(1)
    call PGSCI(14)
    call PGBOX('G',0.,0,'G',0.0,0)
    call PGSCI(1)
    CALL PGBOX('ABCTSN',0.0,0,'ABCTSNV',0.0,0)
    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select
    call pglab(xla,yla,tit)
!    call PGSCI(48)
    call PGSCI(46)
    do i = 1, size(y,2)
       call pgline(size(y,1),x,y(:,i)) 
    enddo
    call PGSCI(14)
    call PGBOX('G',0.,0,'G',0.0,0)
    call PGSCI(1)
    call PGBOX('ABCTSN',0.0,0,'ABCTSNV',0.0,0)

  end subroutine plot_multiline
!__________________________________________________________________

subroutine plot_init(device,xwinsize,ratio,ipan,jpan,font)
  ! dummy variables
    character(len=30), intent(in) :: device
    real, intent(in), optional :: xwinsize, ratio
    integer, intent(in), optional :: ipan, jpan, font
    ! inner variables
    integer :: pgopen, L, C1, C2, NC,ci1,ci2, istat
    real :: contra, bright, c, s
    character(len=16) :: VAL
    
    ISTAT = PGOPEN(trim(device))
    write(*,*) 'pgopen: Istat =', Istat
    IF (ISTAT .LE. 0 ) then
       ISTAT = PGOPEN('?') 
    endif
    IF (ISTAT .LE. 0 ) stop 

    CALL PGQINF('TYPE', VAL, L)
    CALL PGQCIR(C1, C2)
    call PGQCOL (CI1, CI2)
    NC = MAX(0, C2-C1+1)
    IF (NC .LT.8) THEN
       WRITE (*,*) 'Not enough colors available on this device'
       STOP
    ELSE
       WRITE (*,*)
    END IF

    if ((present(ipan)).and.(present(jpan))) then
       call pgsubp(ipan,jpan)
    else
       call pgsubp(1,1)
    endif
    if ((present(xwinsize)).and.(present(ratio))) then
       call PGPAP(xwinsize,ratio)
       write(*,*) ' set xwinsize & ratio '
   elseif((present(xwinsize)).and..not.(present(ratio)))then
       call PGPAP(xwinsize,1.)
    else
       call PGPAP(10.,1.)       
    endif
    if (present(font)) call PGSCF (font)
!Set the Character Font for subsequent text plotting.
!  1: (default) a simple single-stroke font, 2: roman font, 3: italic font,  4: script font
!    call pgscf(1)
    call pgsch(2.)
    call pgsave

end subroutine plot_init
!__________________________________________________________________

subroutine plot_vectfield(field1,field2,i1,i2,j1,j2,xlab,ylab,zlab,title,norm)

  implicit none

  real, dimension(:,:), intent(in) :: field1,field2
  integer, intent(in) :: i1,i2,j1,j2
  character(len=30), optional :: xlab, ylab, zlab, title
  integer, intent(in), optional :: norm 

  integer :: MAXX, MAXY
  real :: TR(6), tempch

  MAXX = size(field1,1)
  MAXY = size(field1,2)
  TR(1) = 1.0
  TR(2) = 1.0
  TR(3) = 0.0
  TR(4) = 1.0
  TR(5) = 0.0
  TR(6) = 1.0
  CALL PGSVP (0.15, 0.85, 0.15, 0.85)
  CALL PGWNAD(real(i1), real(i2), real(j1), real(j2))
  CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
  CALL PGLAB (xlab,ylab,title)
  CALL PGSAH (2, 45.0, 0.7)
  CALL PGQCH (TEMPCH)
  CALL PGSCH (0.3)
  CALL PGVECT (field1,field2,maxx,maxy,i1,i2,j1,j2,0.0,0,TR,1.e31)
  CALL PGSCH (TEMPCH)

end subroutine plot_vectfield

!__________________________________________________________________

  subroutine plot_image(field,xlab,ylab,zlab,title,colormap,fieldmax,box)
    ! dummy variables
    real, dimension(:,:), intent(in) :: field
    character(len=30), optional :: xlab, ylab, zlab, title
    integer, intent(in), optional :: colormap,box 
    real, intent(in), optional :: fieldmax
    ! inner variables
    character(len=30):: xla, yla, tit
    integer :: mxi, mxj, L, C1, C2, NC, idefault, ifont
    real :: fmin, fmax, contra, bright, angle, c, s
    real, dimension(1:6) :: TR(6)
    character(len=16) :: VAL
    character(len=10), parameter :: routine_name = 'plot_image'

    select case(present(xlab));case(.false.);xla='';case(.true.);xla=xlab;end select	
    select case(present(ylab));case(.false.);yla='';case(.true.);yla=ylab;end select	
    select case(present(title));case(.false.);tit='';case(.true.);tit=title;end select

    mxi = size(field,1)
    mxj = size(field,2)

    if (present(fieldmax)) then
       fmin = - fieldmax
       fmax = fieldmax
       else
          fmax =  maxval(abs(field))
          fmin = -fmax
    endif

    TR(1) = 0.0
    TR(2) = 1.0
    TR(3) = 0.0
    TR(4) = 0.0
    TR(5) = 0.0
    TR(6) = 1.0

    if (mxi<mxj) then
       CALL PGSVP (0.15, 0.89, 0.15, 0.85)
    else
       CALL PGSVP (0.15, 0.89, 0.25, 0.85)
    endif
    CALL PGWNAD(0.0, 1.0+MXI, 0.0, 1.0+MXJ)
    if (present(colormap)) then
       BRIGHT = 0.50
       CONTRA  = 1.0
       CALL PALETT(colormap, CONTRA, BRIGHT)
    else
       BRIGHT = 0.50
       CONTRA  = 1.0
       idefault = 6
       CALL PALETT(idefault, CONTRA, BRIGHT)
    endif
    
    CALL PGIMAG(Field,MXI,MXJ,1,MXI,1,MXJ,FMIN,FMAX,TR)
    if(present(title)) then
       tit=title
       CALL PGMTXT('t',1.0,0.5,0.5,title)
    else
       tit=' '
    endif
    if(present(box)) then
	if(box==1)then; CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
        elseif(box==0)then 
        endif
    else
	CALL PGBOX('bcntsi',0.0,0,'bcntsiv',0.0,0)
    endif 
    if (mxi<mxj) then
       if(present(zlab))then
          CALL PGWEDG('RI', 0.75, 3., FMIN, FMAX, zlab)
       else
          CALL PGWEDG('RI', 0.75, 3., FMIN, FMAX,'')
       endif
    else
       if(present(zlab))then
          CALL PGWEDG('BI', 2., 3., FMIN, FMAX, zlab)
       else
          CALL PGWEDG('BI', 2., 3., FMIN, FMAX,'')
       endif
    endif
 
  end subroutine plot_image

!--------------------------------------------------------------------

  
  SUBROUTINE PALETT(TYPE, CONTRA, BRIGHT)
!C-----------------------------------------------------------------------
!C Set a "palette" of colors in the range of color indices used by
!C PGIMAG.
!C-----------------------------------------------------------------------
      INTEGER TYPE
      REAL CONTRA, BRIGHT
!C
      REAL GL(2), GR(2), GG(2), GB(2)
      REAL GiL(2), GiR(2), GiG(2), GiB(2)
      REAL sL(3), sR(3), sG(3), sB(3)
      REAL xL(4), xR(4), xG(4), xB(4)
      REAL RL(9), RR(9), RG(9), RB(9)
      REAL HL(5), HR(5), HG(5), HB(5)
      REAL WL(10), WR(10), WG(10), WB(10)
      REAL AL(20), AR(20), AG(20), AB(20)
!C
      DATA GL /0.0, 1.0/
      DATA GR /0.0, 1.0/
      DATA GG /0.0, 1.0/
      DATA GB /0.0, 1.0/
!c
      DATA GiL /0.0, 1.0/
      DATA GiR /1.0, 0.0/
      DATA GiG /1.0, 0.0/
      DATA GiB /1.0, 0.0/
!c
      DATA sL /0.0, 0.5, 1.0/
      DATA sR /1.0, 0.9, 0.0/
      DATA sG /0.0, 1.0, 0.0/
      DATA sB /0.0, 0.9, 1.0/

      DATA xL /0.0, 0.51, 0.51 , 1.0/
      DATA xR /0.0, 0.0, 0.0 , 0.0/
      DATA xG /0.0, 0.0, 0.8 , 0.8/
      DATA xB /0.0, 0.0, 0.0 , 0.0/

!      DATA sL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
!      DATA sR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
!      DATA sG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
!      DATA sB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
!C
      DATA RL /-0.5, 0.0, 0.17, 0.33, 0.50, 0.67, 0.83, 1.0, 1.7/
      DATA RR / 0.0, 0.0,  0.0,  0.0,  0.6,  1.0,  1.0, 1.0, 1.0/
      DATA RG / 0.0, 0.0,  0.0,  1.0,  1.0,  1.0,  0.6, 0.0, 1.0/
      DATA RB / 0.0, 0.3,  0.8,  1.0,  0.3,  0.0,  0.0, 0.0, 1.0/
!C
!      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
!      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
!      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
!      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/

      DATA HL /0.0, 0.2, 0.4, 0.6, 1.0/
      DATA HR /0.0, 0.5, 1.0, 1.0, 1.0/
      DATA HG /0.0, 0.0, 0.5, 1.0, 1.0/
      DATA HB /0.0, 0.0, 0.0, 0.3, 1.0/
!C
      DATA WL /0.0, 0.5, 0.5, 0.7, 0.7, 0.85, 0.85, 0.95, 0.95, 1.0/
      DATA WR /0.0, 1.0, 0.0, 0.0, 0.3,  0.8,  0.3,  1.0,  1.0, 1.0/
      DATA WG /0.0, 0.5, 0.4, 1.0, 0.0,  0.0,  0.2,  0.7,  1.0, 1.0/
      DATA WB /0.0, 0.0, 0.0, 0.0, 0.4,  1.0,  0.0,  0.0, 0.95, 1.0/
!C
      DATA AL /0.0, 0.1, 0.1, 0.2, 0.2, 0.3, 0.3, 0.4, 0.4, 0.5,&
     &         0.5, 0.6, 0.6, 0.7, 0.7, 0.8, 0.8, 0.9, 0.9, 1.0/
      DATA AR /0.0, 0.0, 0.3, 0.3, 0.5, 0.5, 0.0, 0.0, 0.0, 0.0,&
     &         0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0/
      DATA AG /0.0, 0.0, 0.3, 0.3, 0.0, 0.0, 0.0, 0.0, 0.8, 0.8,&
     &         0.6, 0.6, 1.0, 1.0, 1.0, 1.0, 0.8, 0.8, 0.0, 0.0/
      DATA AB /0.0, 0.0, 0.3, 0.3, 0.7, 0.7, 0.7, 0.7, 0.9, 0.9,&
     &         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/
!C
      IF (TYPE.EQ.1) THEN
!C        -- gray scale
         CALL PGCTAB(GL, GR, GG, GB, 2, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.2) THEN
!C        -- rainbow
         CALL PGCTAB(RL, RR, RG, RB, 9, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.3) THEN
!C        -- heat
         CALL PGCTAB(HL, HR, HG, HB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.4) THEN
!C        -- weird IRAF
         CALL PGCTAB(WL, WR, WG, WB, 10, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.5) THEN
!C        -- AIPS
         CALL PGCTAB(AL, AR, AG, AB, 20, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.6) THEN
!C        -- AIPS
         CALL PGCTAB(sL, sR, sG, sB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.7) THEN
!C        -- AIPS
         CALL PGCTAB(xL, xR, xG, xB, 5, CONTRA, BRIGHT)
      ELSE IF (TYPE.EQ.8) THEN
!C        -- AIPS
         CALL PGCTAB(GiL, GiR, GiG, GiB, 2, CONTRA, BRIGHT)
      END IF
      END subroutine

      SUBROUTINE SETVP
!C-----------------------------------------------------------------------
!C Set the viewport, allowing margins around the edge for annotation.
!C (This is similar in effect to PGVSTD, but has different margins.)
!C The routine determines the view-surface size and allocates margins
!C as fractions of the minimum of width and height.
!C-----------------------------------------------------------------------
      REAL D, VPX1, VPX2, VPY1, VPY2
!C
      CALL PGSVP(0.0, 1.0, 0.0, 1.0)
      CALL PGQVP(1, VPX1, VPX2, VPY1, VPY2)
      D = MIN(VPX2-VPX1, VPY2-VPY1)/40.0
      VPX1 = VPX1 + 5.0*D
      VPX2 = VPX2 - 2.0*D
      VPY1 = VPY1 + 8.0*D
      VPY2 = VPY2 - 2.0*D
      CALL PGVSIZ(VPX1, VPX2, VPY1, VPY2)
      END subroutine

      SUBROUTINE FIDDLE
!C
      INTEGER P, IER, PGCURS
      REAL CONTRA, BRIGHT, X, Y, SIGN
      REAL X1, Y1, X2, Y2, B1, B2, C1, C2
      CHARACTER CH
!C
      WRITE (*,*) 'Use cursor to adjust color table:'
      WRITE (*,*) ' Keys 1,2,3,4,5 select different palettes'
      WRITE (*,*) ' Key P cycles through available palettes'
      WRITE (*,*) ' Key F adjusts contrast and brightness, with'
      WRITE (*,*) '  cursor x position setting brightness [0.0 - 1.0]'
      WRITE (*,*) '   and y position setting contrast [0.0 - 10.0]'
      WRITE (*,*) '  (Hold down F key while moving cursor to change'
      WRITE (*,*) '  contrast and brightness continuously)'
      WRITE (*,*) ' Key C resets contrast=1.0, brightness=0.5'
      WRITE (*,*) ' Key - reverses color palette'
      WRITE (*,*) ' Key X or right mouse button exits program' 
!C
      P = 2
      CONTRA = 1.0
      BRIGHT = 0.5
      X = 0.5
      Y = 1.0
      SIGN = +1.0
!C
      CALL PGQWIN(X1, X2, Y1, Y2)
      B1 = 0.0
      B2 = 1.0
      C1 = 0.0
      C2 = 10.0
      CALL PGSWIN(B1, B2, C1, C2)
 10   IER = PGCURS(X, Y, CH)
      IF (CH.EQ.CHAR(0) .OR. CH.EQ.'x' .OR. CH.EQ.'X') THEN
         CALL PGSWIN(X1, X2, Y1, Y2)
         RETURN
      ELSE IF (CH.EQ.'F' .OR. CH.EQ.'f') THEN
         BRIGHT = MAX(B1, MIN(B2,X))
         CONTRA = MAX(C1, MIN(C2,Y))
      ELSE IF (CH.EQ.'C' .OR. CH.EQ.'c') THEN
         CONTRA = 1.0
         Y = 1.0
         BRIGHT = 0.5
         X = 0.5
      ELSE IF (CH.EQ.'-') THEN
         SIGN = -SIGN
      ELSE IF (CH.EQ.'1') THEN
         P = 1
      ELSE IF (CH.EQ.'2') THEN
         P = 2
      ELSE IF (CH.EQ.'3') THEN
         P = 3
      ELSE IF (CH.EQ.'4') THEN
         P = 4
      ELSE IF (CH.EQ.'5') THEN
         P = 5
      ELSE IF (CH.EQ.'P' .OR. CH.EQ.'p') THEN
         P = 1 + MOD(P,5)
      END IF
      CALL PALETT(P, SIGN*CONTRA, BRIGHT)
      GOTO 10
      END subroutine

      SUBROUTINE OUTLIN(I1,I2,J1,J2,TR)
      INTEGER I1,I2,J1,J2
      REAL TR(6)
!C-----------------------------------------------------------------------
!C Draw the enclosing rectangle of the subarray to be contoured,
!C applying the transformation TR.
!C
!C For a contour map, the corners are (I1,J1) and (I2,J2); for
!C a gray-scale map, they are (I1-0.5,J1-0.5), (I2+0.5, J2+0.5).
!C-----------------------------------------------------------------------
      INTEGER K
      REAL XW(5), YW(5), T
!C
      XW(1) = I1
      YW(1) = J1
      XW(2) = I1
      YW(2) = J2
      XW(3) = I2
      YW(3) = J2
      XW(4) = I2
      YW(4) = J1
      XW(5) = I1
      YW(5) = J1
      DO 10 K=1,5
          T = XW(K)
          XW(K) = TR(1) + TR(2)*T + TR(3)*YW(K)
          YW(K) = TR(4) + TR(5)*T + TR(6)*YW(K)
   10 CONTINUE
      CALL PGLINE(5,XW,YW)

    end subroutine

!function pglnblnk(c)
!
!  implicit none
!  	
!  integer :: pglnblnk
!  integer :: i,ii
!  character(len=*):: c
!  
!  ii=0
!  do i=1,len(c)
!     if(c(i:i) ==' ')ii=ii+1
!  enddo
!  pglnblnk=len(c)-ii
!  
!end function pglnblnk

!C*PGVECT -- vector map of a 2D data array, with blanking
!C%void cpgvect(const float *a, const float *b, int idim, int jdim, \
!C% int i1, int i2, int j1, int j2, float c, int nc, \
!C% const float *tr, float blank);
!C+
SUBROUTINE PGVECT2 (A, B, IDIM, JDIM, I1, I2, J1, J2, C, NC, TR,BLANK)
  INTEGER IDIM, JDIM, I1, I2, J1, J2, NC
  REAL    A(IDIM,JDIM), B(IDIM, JDIM), TR(6), BLANK, C
  !C
  !C Draw a vector map of two arrays.  This routine is similar to
  !C PGCONB in that array elements that have the "magic value" defined by
  !C the argument BLANK are ignored, making gaps in the vector map.  The
  !C routine may be useful for data measured on most but not all of the
  !C points of a grid. Vectors are displayed as arrows; the style of the
  !C arrowhead can be set with routine PGSAH, and the the size of the
  !C arrowhead is determined by the current character size, set by PGSCH.
  !C
  !C Arguments:
  !C  A      (input)  : horizontal component data array.
  !C  B      (input)  : vertical component data array.
  !C  IDIM   (input)  : first dimension of A and B.
  !C  JDIM   (input)  : second dimension of A and B.
  !C  I1,I2  (input)  : range of first index to be mapped (inclusive).
  !C  J1,J2  (input)  : range of second index to be mapped (inclusive).
  !C  C      (input)  : scale factor for vector lengths, if 0.0, C will be
  !C                    set so that the longest vector is equal to the
  !C                    smaller of TR(2)+TR(3) and TR(5)+TR(6).
  !C  NC     (input)  : vector positioning code.
  !C                    <0 vector head positioned on coordinates
  !C                    >0 vector base positioned on coordinates
  !C                    =0 vector centered on the coordinates
  !C  TR     (input)  : array defining a transformation between the I,J
  !C                    grid of the array and the world coordinates. The
  !C                    world coordinates of the array point A(I,J) are
  !C                    given by:
  !C                      X = TR(1) + TR(2)*I + TR(3)*J
  !C                      Y = TR(4) + TR(5)*I + TR(6)*J
  !C                    Usually TR(3) and TR(5) are zero - unless the
  !C                    coordinate transformation involves a rotation
  !C                    or shear.
  !C  BLANK   (input) : elements of arrays A or B that are exactly equal to
  !C                    this value are ignored (blanked).
  !C--
  !C  4-Sep-1992: derived from PGCONB [J. Crane].
  !C 26-Nov-1992: revised to use PGARRO [TJP].
  !C 25-Mar-1994: correct error for NC not =0 [G. Gonczi].
  !C  5-Oct-1996: correct error in computing max vector length [TJP;
  !C              thanks to David Singleton].
  !C-----------------------------------------------------------------------
  INTEGER  I, J
  REAL X, Y, X1, Y1, X2, Y2
  REAL CC
  INTRINSIC SQRT, MAX, MIN

!  write(*,*) 'begin of pgvect2'

  !C
  !C Define grid to world transformation
  !C
  X(I,J) = TR(1) + TR(2)*I + TR(3)*J
  Y(I,J) = TR(4) + TR(5)*I + TR(6)*J
  write(*,*) 'begin of pgvect2'
  !C
  !C Check arguments.
  !C
  IF (I1.LT.1 .OR. I2.GT.IDIM .OR. I1.GE.I2 .OR.J1.LT.1 .OR. J2.GT.JDIM .OR. J1.GE.J2) THEN
     !C        CALL GRWARN('PGVECT: invalid range I1:I2, J1:J2')
     RETURN
  END IF
  !C
  !C Check for scale factor C.
  !C
  CC = C
  IF (CC.EQ.0.0) THEN
     DO 20 J=J1,J2
        DO 10 I=I1,I2
           IF (A(I,J).NE.BLANK .AND. B(I,J).NE.BLANK) CC = MAX(CC,SQRT(A(I,J)**2+B(I,J)**2))
10         CONTINUE
20         CONTINUE
           IF (CC.EQ.0.0) RETURN
           CC = SQRT(MIN(TR(2)**2+TR(3)**2,TR(5)**2+TR(6)**2))/CC
        END IF
        !C
        !c      CALL PGBBUF
        !C
        DO 40 J=J1,J2
           DO 30 I=I1,I2
              !C
              !C Ignore vector if element of A and B are both equal to BLANK
              !C
              IF (.NOT.(A(I,J).EQ.BLANK .AND. B(I,J).EQ.BLANK)) THEN

                 !C
                 !C Define the vector starting and end points according to NC.
                 !C
                 IF (NC.LT.0) THEN
                    X2 = X(I,J)
                    Y2 = Y(I,J)
                    X1 = X2 - A(I,J)*CC
                    Y1 = Y2 - B(I,J)*CC
                 ELSE IF (NC.EQ.0) THEN
                    X2 = X(I,J) + 0.5*A(I,J)*CC
                    Y2 = Y(I,J) + 0.5*B(I,J)*CC
                    X1 = X2 - A(I,J)*CC
                    Y1 = Y2 - B(I,J)*CC
                 ELSE
                    X1 = X(I,J)
                    Y1 = Y(I,J)
                    X2 = X1 + A(I,J)*CC
                    Y2 = Y1 + B(I,J)*CC
                 END IF
                 !C     
                 !C Draw vector.
                 !C
                 CALL PGARRO(X1, Y1, X2, Y2)
              END IF
30            CONTINUE
40            CONTINUE
              !C
              !c      CALL PGEBUF
              write(*,*) 'end of pgvect2'
            END subroutine pgvect2!end module plot_mod











