PROGRAM grandx
USE gcarcs
IMPLICIT NONE

REAL, DIMENSION(3,16020) :: layer
REAL, DIMENSION(2,180) :: xpoint
REAL :: stlat, stlon, evlat, evlon, Rad
REAL :: x1, y1, z1, x2, y2, z2
REAL :: p1, p2, p3, alpha, s, dprod
REAL :: xo, yo, zo, rot, Radt, Latt, Lont
REAL :: xlat, xlon, glat, glon, xmindis 
REAL :: mindepth, maxdepth, meandepth, vclose
INTEGER :: J, K, L
CHARACTER(len=50) :: infile
CHARACTER(len=20)  :: lstring


!=======================================================================!
! Set Station and Event Latitudes and Longitudes
!   (note longitudes must be between 0-360 deg

stlat = 33.49
stlon = 243.33
evlat = -13.74
evlon = 291.21

!=======================================================================!

Rad = 6371.0

!Convert Ray Endpoints to cartesian coordinates
Call cartesian(evlat, evlon, Rad, x1, y1, z1)
Call cartesian(stlat, stlon, Rad, x2, y2, z2)

!Determine Pole of Rotation, and angle between vectors
!p1,p2,p3 = <x,y,z> unit vector of pole of rotation  
Call cross(x1, y1, z1, x2, y2, z2, p1, p2, p3, alpha, s)
Call dot(x1, y1, z1, x2, y2, z2, dprod, alpha, s)

!We next want to find the point that is 180 deg away
!from the event lat, lon on the gcarc between the
!event and station (Latt, Lont)
rot = 180.0   !rotation of 180 deg
Call euler(x1,y1,z1,p1,p2,p3,rot,xo,yo,zo)
Call latlon(xo, yo, zo, Latt, Lont, Radt)

!Now lets find 180 lat, lon pairs along the gcarc between
!the event and the antipode
rot = 0.0
DO L=1,181
  Call euler(x1,y1,z1,p1,p2,p3,rot,xo,yo,zo)
  Call latlon(xo, yo, zo, Latt, Lont, Radt)
  xpoint(1,L) = Latt
  xpoint(2,L) = Lont
  rot = rot + 1.0
ENDDO


!read in the grand model
!grand model contains 22-layers named out1-out22
!format is lat, lon, velocity
!Note:  vel=999 is steve's undefined block

OPEN(UNIT=3,FILE='Base_xsctn_grand2002')
DO J=1,22    !Loop through layers

  write(*,FMT="(A1,A,t20,I3)",ADVANCE="NO") achar(13), &
  &  "Working on Layer: ", J

  write(lstring,*) J
  lstring = adjustl(lstring)
  infile = './Grand_2002/out'//TRIM(lstring(1:2)) 

  OPEN(UNIT=1,FILE=infile)
  read(1,*) mindepth, maxdepth
  DO K=1,16020
    read(1,*) layer(1,K), layer(2,K), layer(3,K)
  ENDDO
  CLOSE(1)

  meandepth = mindepth + (maxdepth - mindepth)/2.
  !Now we have to find the closest velocity in grand's
  !to the xpoint's
  Rad = 6371. - meandepth
  DO L=1,181
    xlat = xpoint(1,L)
    xlon = xpoint(2,L)
    Call cartesian(xlat, xlon, Rad, x1, y1, z1)
    xmindis = 100000.
    vclose = 999.
    DO K=1,16020
      glat = layer(1,K)
      glon = layer(2,K)
      Call cartesian(glat, glon, Rad, x2, y2, z2)
      Call dot(x1, y1, z1, x2, y2, z2, dprod, alpha, s)
      IF (alpha < xmindis) THEN
        xmindis = alpha
        vclose = layer(3,K)
      ENDIF
    ENDDO
    !would like to write theta, radius, velocity
    write(3,*) float(L-1), Rad, vclose
    !write(3,*) xlat, xlon, Rad, vclose
 
  ENDDO
ENDDO

CLOSE(3)

END PROGRAM grandx
