#!/bin/csh
#
#  Plot cross-section derived from Grand_Xsect.f90
#  Requires the Taup toolkit for raytracing.
#
#Set Variables
#~~~~~~~~~~~~~

set depth      = (200)                        #source depth (km)
set distance   = (40.0 50.0 60.0)             #distance to receiver (deg)
set azimuth    = 90                           #{0-360} maximum range of plot in degrees
set phase_list = (S ScS)                      #show rays for seismic phases
set file       = Base_xsctn_grand2002         #Base xsction from Grand_Xsect.f90
set title      = 'X-section: tomo_01 '        #plot title
set title2     = 'Receivers: 40, 50, 60 deg'  #plot subtitle
set output     = tomo_01.ps                   #output filename of plot

xyz2grd $file -Ggrand.grd -R0/180/3480/6380 -I1/10
grdimage grand.grd -Ccolor_redblue.cpt -Y2 -R0/${azimuth}/3480/6371 -JP7i \
  -P -V -K -B10g1000/500g100000NseW >! xsect.ps

foreach phase ($phase_list)
  foreach dist ($distance)
    taup_path -mod prem -h $depth -deg $dist -ph $phase
    awk 'NR > 1 {print $1, $2}' taup_path.gmt >! $phase.xy
    psxy $phase.xy -JP -R -P -K -O >> xsect.ps
  end
  rm $phase.xy
end

psscale -Ccolor_redblue.cpt -L -D3.5/-0.5/3.0/0.3h -B1 -K -P -O >> xsect.ps

pstext -R0/8/0/11 -JX8i/11i -P -O -N << END >> xsect.ps
.25 8.0 10 0 1 1 $title 
.25 7.75 10 0 1 1 $title2
3.55 -1.5 14 0 12 1 d
3.7 -1.5 14 0 1 1 Vs (%)
END


rm grand.grd
rm taup_path.gmt

mv xsect.ps $output

gs $output

