PROGRAM stimefunc
USE sac_i_o
IMPLICIT NONE
REAL, DIMENSION(:,:), ALLOCATABLE :: wavelet
REAL, DIMENSION(:), ALLOCATABLE :: sacfile, dummy
REAL, PARAMETER :: pi=3.141592653589793
REAL    :: Tdom, delta1, sigma, arg, t
REAL    :: extrema, mm, rdelta, Ts
INTEGER :: NN, npts1, J
CHARACTER(LEN=16)  :: stype
CHARACTER(LEN=104) :: input, file1, ofile


!READ COMMAND LINE ARGUMENTS
!========================================================================!
NN = IARGC()
IF (NN < 3) THEN
  write(*,'(a)') "usage:  sfun filename type Tdom"
  write(*,'(a)') "  Types : "
  write(*,'(a)') "  gauss : gaussian wavelet"
  write(*,'(a)') " dgauss : derivative of gaussian wavelet"
  write(*,'(a)') " ricker : ricker wavelet"
  write(*,'(a)') " kupper : kupper wavelet"
  write(*,'(a)') "  refl1 : reflectivity 1st order"
  write(*,'(a)') "  refl2 : reflectivity 2nd order"
  write(*,'(a)') "  refl3 : reflectivity 3rd order"
  write(*,'(a)') "    box : box car function"
  STOP
ENDIF

CALL GETARG(1,input)
file1 = input
write(*,*) "file1= ", TRIM(file1)

CALL GETARG(2,input)
stype = input
write(*,*) "stype= ", TRIM(stype)

CALL GETARG(3,input)
READ(input,*) Tdom
write(*,*) "Tdom= ", Tdom


!READ file to get out delta1 and npts1
!========================================================================!

!read file1
CALL rasac(file1,delta1,depmin,depmax,scale,odelta,b,e,o,a,internal1,        &
t0,t1,t2,t3,t4,t5,t6,t7,t8,t9,f,resp0,resp1,resp2,resp3,resp4,resp5,resp6,   &
resp7,resp8,resp9,stla,stlo,stel,stdp,evla,evlo,evel,evdp,mag,user0,user1,   &
user2,user3,user4,user5,user6,user7,user8,user9,dist,az,baz,gcarc,internal2, &
internal3,depmen,cmpaz,cmpinc,xminimum,xmaximum,yminimum,ymaximum,unused1,   &
unused2,unused3,unused4,unused5,unused6,unused7,nzyear,nzjday,nzhour,nzmin,  &
nzsec,nzmsec,nvhdr,norid,nevid,npts1,internal4,nwfid,nxsize,nysize,unused8,  &
iftype,idep,iztype,unused9,iinst,istreg,ievreg,ievtyp,iqual,isynth,imagtyp,  &
imagsrc,unused10,unused11,unused12,unused13,unused14,unused15,unused16,      &
unused17,leven,lpspol,lovrok,lcalda,unused18,kevnm,kstnm,khole,ko,ka,kt0,kt1,&
kt2,kt3,kt4,kt5,kt6,kt7,kt8,kt9,kf,kuser0,kuser1,kuser2,kcmpnm,knetwk,kdatrd,&
kinst,sacfile)

IF (nvhdr /= 6) THEN
  write(*,*) "ERROR - File: '", TRIM(adjustl(file1)), "' appears to be of non-native &
  &byte-order or is not a SAC file."
  STOP
ENDIF

!what we need from this is delta1 = dt increment
!     and npts1 = number of points


!Create source time function
!========================================================================!
ALLOCATE(wavelet(2,-npts1/2:npts1/2))
sigma = Tdom/(2*pi)

IF (stype == 'gauss') THEN
  arg = 1/SQRT((2*pi*sigma**2))
  DO J=0,npts1/2
    t = REAL(delta1*J)
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    wavelet(2,J) = arg*exp(-(t**2)/(2*sigma**2))
    wavelet(2,-J) = arg*exp(-((-t)**2)/(2*sigma**2))
  ENDDO

ELSEIF (stype == 'dgauss') THEN
  arg = -1/((sigma**2)*SQRT(2*pi*sigma**2))
  DO J=0,npts1/2
    t = REAL(delta1*J)
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    wavelet(2,J) = t*arg*exp(-(t**2)/(2*sigma**2))
    wavelet(2,-J) = -t*arg*exp(-((-t)**2)/(2*sigma**2))
  ENDDO

ELSEIF (stype == 'refl1') THEN
  extrema = 1
  rdelta=(extrema*pi)/Tdom
  mm = (extrema+2.0)/extrema
  DO J=0,npts1/2
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    t= REAL(delta1*J)
    IF (t >= -Tdom/2 .and. t <= Tdom/2) THEN
     wavelet(2,J)=SIN((rdelta*(t+Tdom/2)))-(1/mm)*SIN((mm*rdelta*(t+Tdom/2)))
     wavelet(2,-J)=wavelet(2,J)
    ENDIF
  ENDDO

ELSEIF (stype == 'refl2') THEN
  extrema = 2
  rdelta=(extrema*pi)/Tdom
  mm = (extrema+2.0_8)/extrema
  DO J=0,npts1/2
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    t= REAL(delta1*J)
    IF (t >= -Tdom/2 .and. t <= Tdom/2) THEN
      wavelet(2,J)=SIN((rdelta*(t+Tdom/2)))-(1/mm)*SIN((mm*rdelta*(t+Tdom/2)))
      wavelet(2,-J)=-wavelet(2,J)
    ENDIF
  ENDDO

ELSEIF (stype == 'refl3') THEN
  extrema = 3
  rdelta=(extrema*pi)/Tdom
  mm = (extrema+2.0_8)/extrema
  DO J=0,npts1/2
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    t= REAL(delta1*J)
    IF (t >= -Tdom/2 .and. t <= Tdom/2) THEN
      wavelet(2,J)=SIN((rdelta*(t+Tdom/2)))-(1/mm)*SIN((mm*rdelta*(t+Tdom/2)))
      wavelet(2,-J)=wavelet(2,J)
    ENDIF
  ENDDO

ELSEIF (stype == 'box') THEN
  DO J=0,npts1/2
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    t=REAL(delta1*J)
    IF (t >= - Tdom/2 .and. t <= Tdom/2) THEN
      wavelet(2,J) = 1.0_8
      wavelet(2,-J) = 1.0_8
    ELSE
      wavelet(2,J) = 0.0_8
      wavelet(2,-J) = 0.0_8
    ENDIF
  ENDDO

ELSEIF (stype == 'kupper') THEN
  DO J=0,npts1/2
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    t=REAL(delta1*J)
    wavelet(2,-J) = 0.0
    IF (t <= Tdom) THEN
      wavelet(2,J) = sin((2.0*pi*t/Tdom))-0.5*sin((4.0*pi*t/Tdom))
    ELSE
      wavelet(2,J) = 0.0
    ENDIF
  ENDDO

ELSEIF (stype == 'ricker') THEN
  Ts = 1.1*Tdom
  DO J=0,npts1/2
    wavelet(1,J) = delta1*J
    wavelet(1,-J) = -delta1*J
    t=REAL(delta1*J)
    arg = ((pi*(t-Ts)/Tdom))**2
    wavelet(2,-J) = 0.0
    IF (t <= 2.0*Ts) THEN
      wavelet(2,J) = (0.5*sqrt(pi))*(arg-0.5)*exp((-arg))
    ELSE
      wavelet(2,J) = 0.0
    ENDIF
  ENDDO

ENDIF

!normalize to 1
wavelet(2,:) = wavelet(2,:)/MAXVAL(wavelet(2,:))


!Write Source-time function to a SAC file
!========================================================================!
!Initialize sac file
CALL initsac(ofile,delta,depmin,depmax,scale,odelta,b,e,o,a,internal1,       &
t0,t1,t2,t3,t4,t5,t6,t7,t8,t9,f,resp0,resp1,resp2,resp3,resp4,resp5,resp6,   &
resp7,resp8,resp9,stla,stlo,stel,stdp,evla,evlo,evel,evdp,mag,user0,user1,   &
user2,user3,user4,user5,user6,user7,user8,user9,dist,az,baz,gcarc,internal2, &
internal3,depmen,cmpaz,cmpinc,xminimum,xmaximum,yminimum,ymaximum,unused1,   &
unused2,unused3,unused4,unused5,unused6,unused7,nzyear,nzjday,nzhour,nzmin,  &
nzsec,nzmsec,nvhdr,norid,nevid,npts,internal4,nwfid,nxsize,nysize,unused8,   &
iftype,idep,iztype,unused9,iinst,istreg,ievreg,ievtyp,iqual,isynth,imagtyp,  &
imagsrc,unused10,unused11,unused12,unused13,unused14,unused15,unused16,      &
unused17,leven,lpspol,lovrok,lcalda,unused18,kevnm,kstnm,khole,ko,ka,kt0,kt1,&
kt2,kt3,kt4,kt5,kt6,kt7,kt8,kt9,kf,kuser0,kuser1,kuser2,kcmpnm,knetwk,kdatrd,&
kinst,dummy)

ofile = TRIM(adjustl(stype))//'.sac'
kstnm = TRIM(adjustl(stype))
delta = delta1
depmin = MINVAL(wavelet(2,:))
depmax = MAXVAL(wavelet(2,:))
npts = size(wavelet(2,:))
b = wavelet(1,-npts1/2)
e = wavelet(1,npts1/2)

CALL wbsac(ofile,delta,depmin,depmax,scale,odelta,b,e,o,a,internal1,         &
t0,t1,t2,t3,t4,t5,t6,t7,t8,t9,f,resp0,resp1,resp2,resp3,resp4,resp5,resp6,   &
resp7,resp8,resp9,stla,stlo,stel,stdp,evla,evlo,evel,evdp,mag,user0,user1,   &
user2,user3,user4,user5,user6,user7,user8,user9,dist,az,baz,gcarc,internal2, &
internal3,depmen,cmpaz,cmpinc,xminimum,xmaximum,yminimum,ymaximum,unused1,   &
unused2,unused3,unused4,unused5,unused6,unused7,nzyear,nzjday,nzhour,nzmin,  &
nzsec,nzmsec,nvhdr,norid,nevid,npts,internal4,nwfid,nxsize,nysize,unused8,   &
iftype,idep,iztype,unused9,iinst,istreg,ievreg,ievtyp,iqual,isynth,imagtyp,  &
imagsrc,unused10,unused11,unused12,unused13,unused14,unused15,unused16,      &
unused17,leven,lpspol,lovrok,lcalda,unused18,kevnm,kstnm,khole,ko,ka,kt0,kt1,&
kt2,kt3,kt4,kt5,kt6,kt7,kt8,kt9,kf,kuser0,kuser1,kuser2,kcmpnm,knetwk,kdatrd,&
kinst,wavelet(2,:))



END PROGRAM stimefunc

