#!/bin/csh 
#--------------------------------------------------------------------#
# Standard gmt script for plotting snapshots.
# Author: Michael Thorne (mthorne@asu.edu)
# Last Update:  9/16/2007
#
# Requires ImageMagick (http://www.imagemagick.org/) for JPEG
# compression and resizing images. (Can be commented out if
# you don't have ImageMagick)
#
# Requires the following addition to $GMTHOME/share/gmtmedia.d
# snap        672             504

#--------------------------------------------------------------------#
# Set gmtdefaults:
# ~~~~~~~~~~~~~~~~
#the paper media is specified in the file gmtmedia.d
#located in $GMTHOME/share

#to view current gmt defaults:
#gmtdefaults -D | less

gmtset X_ORIGIN 0
gmtset Y_ORIGIN 0
gmtset PAGE_COLOR 0/0/0
gmtset PAPER_MEDIA snap
gmtset DOTS_PR_INCH 300
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
# Set information for the current snapshot
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
set file = ./.plotinfo
set dr = `awk 'NR == 1 {print $1}' $file`
set dtheta = `awk 'NR == 2 {print $1}' $file`
set mina = `awk 'NR == 3 {print $1}' $file`
set maxa = `awk 'NR == 4 {print $1}' $file`
set minr = `awk 'NR == 5 {print $1}' $file`
set maxr = `awk 'NR == 6 {print $1}' $file`
set rotation = `awk 'NR == 7 {print $1}' $file`
set time = `awk 'NR == 8 {print $1}' $file`
set snap = `awk 'NR == 9 {print $1}' $file`
set otype = `awk 'NR == 10 {print $1}' $file`
set resolution = `awk 'NR == 11 {print $1}' $file`
set gridfile = `awk 'NR == 12 {print $1}' $file` 
set width = `awk 'NR == 13 {print $1}' $file`
set xoffset = `awk 'NR == 14 {print $1}' $file`
set height = `awk 'NR == 15 {print $1}' $file`
set yoffset = `awk 'NR == 16 {print $1}' $file`
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
#  Create color palette table
#  ~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Standard Red/Blue Color Palette Table
echo -1   0    0    255  0  255  255  255 >! color.cpt
echo 0    255  255  255  1  255  0    0 >> color.cpt
echo B    0    0    255 >> color.cpt
echo F    255  0    0 >> color.cpt
echo N    0    0    0 >> color.cpt


#  Shinjuku Color Palette Table
#echo -1   0    0   255   0  0    0    0 >! color.cpt
#echo 0    0    0   0     1  255  0    0 >> color.cpt
#echo B    0    0   255 >> color.cpt
#echo F    255  0   0 >> color.cpt
#echo N    0    0   0 >> color.cpt


#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
#  Create GMT gridfile
#  ~~~~~~~~~~~~~~~~~~~
xyz2grd $gridfile -bis -Gsh${snap}.grd -R-180/180/3480/6371 \
 -V -I${dtheta}/${dr} -Al
#--------------------------------------------------------------------#


#--------------------------------------------------------------------#
#  Add Text
#  ~~~~~~~~
pstext -R0/9.333/0/7 -JX9.333i/7i -N -G255/0/0 -K -P << END >! sh${snap}.ps
.25 .25 16 0 7 1  $time (sec)
6.5 .25 10 0 5 1 [M. Thorne, G. Jahnke, and H. Igel - 2007]
.25 6.5 16 0 5 1 Model:  PREM
END
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
#  Grdimage
#  ~~~~~~~~
grdimage sh${snap}.grd -Ccolor.cpt -R${mina}/${maxa}/${minr}/${maxr} \
 -JP${width}/${rotation} -X${xoffset} -Y${yoffset} -V -Bnsew -P \
 -O -K >> sh${snap}.ps
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
#  Draw discontinuities
#  ~~~~~~~~~~~~~~~~~~~~
psxy /tmp/410disc -JP -R -O -K -W1/0/0/0 -V -P  >> sh${snap}.ps
psxy /tmp/660disc -JP -R -O -W1/0/0/0 -V -P -K  >> sh${snap}.ps
psxy /tmp/laydisc -JP -R -O -W1/0/0/0 -V -P  >> sh${snap}.ps
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
#  Convert to jpeg image (using ImageMagick)
#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
convert -compress Lossless -density ${resolution}x${resolution} \
   sh${snap}.ps shf${snap}.jpg
#--------------------------------------------------------------------#

#--------------------------------------------------------------------#
#  Remove temporary files
#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/bin/rm color.cpt
/bin/rm .gmtdefaults4
/bin/rm .gmtcommands4
/bin/rm $gridfile
/bin/rm sh${snap}.grd
#--------------------------------------------------------------------#

