PROGRAM shsnaps
IMPLICIT NONE
REAL, PARAMETER    :: pi=3.141592654
REAL, DIMENSION(:,:), ALLOCATABLE :: data
REAL               :: mintheta, maxtheta, minrad, maxrad, dt
REAL               :: radius, theta
REAL               :: mina, maxa, minr, maxr, scale
REAL(KIND=8)       :: dr, dtheta
REAL               :: rotation, width, height, xoffset, yoffset
REAL               :: xmax, tscale, ymin, ymax, arg 
REAL               :: A, B
REAL               :: epsilon
INTEGER, PARAMETER :: maxranks = 100
INTEGER            :: xpixels, ypixels 
INTEGER            :: NN
INTEGER            :: J, K, M
INTEGER            :: nranks             !total number of ranks
INTEGER            :: nx, nz             !grid spacing
INTEGER            :: iout, it 
INTEGER            :: ios
INTEGER            :: count, nnnx, resolution
INTEGER            :: pid, pidlen
CHARACTER(LEN=5)   :: region, input, otype   !
CHARACTER(LEN=3)   :: snap
CHARACTER(LEN=6)   :: pidstr
CHARACTER(LEN=100) :: cmd, snapfile
CHARACTER(LEN=18)  :: junk

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
!Read input from command line
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~
!usage:  ./shnap snap_number 

NN = IARGC()     !determine the total number of command line args given.

IF (NN < 1) THEN
  write(*,*) "usage:  ./shnap snap_number"
  STOP
ENDIF

!Read the first argument (snap)
!E.g., snap number is 003, or 022, or 054
CALL GETARG(1, snap)

pid = getpid()
IF (pid < 10) THEN
  pidlen = 1
ELSEIF (pid < 100) THEN
  pidlen = 2
ELSEIF (pid < 1000) THEN
  pidlen = 3
ELSEIF (pid < 10000) THEN
  pidlen = 4
ELSEIF (pid < 100000) THEN
  pidlen = 5
ELSE
  pidlen = 6
ENDIF
write(pidstr,'(I6.6)') pid 
write(*,*) "PID= ", TRIM(pidstr(2:(pidlen+1)))
!write(*,*) "PID= ", TRIM(adjustl(pidstr(2:(pidlen+1))))

!Deal with the .defaults file
OPEN(1,FILE='shnapdefaults',STATUS='OLD',IOSTAT=ios)
IF (ios > 0) THEN
  write(*,*) "Error:  no shnapdefaults file in working directory..."
  write(*,*) "Exiting..."
  STOP
ENDIF

!Read input from defaults
 READ(1,*) junk
 READ(1,*) junk
 READ(1,1) junk, resolution
 READ(1,*) junk
 READ(1,2) junk, minr
 READ(1,2) junk, maxr
 READ(1,2) junk, mina
 READ(1,2) junk, maxa
 READ(1,*) junk
 READ(1,2) junk, scale
CLOSE(1)
1 format(a,i8)
2 format(a,e12.6) 

write(*,*) "Minimum Radius:  ", minr, " [km]"
write(*,*) "Maximum Radius:  ", maxr, " [km]"
write(*,*) "Minimum Angle:   ", mina, "[deg]"
write(*,*) "Maximum Angle:   ", maxa, "[deg]"
write(*,*) "Scale factor:    ", scale
write(*,*) "---------------------------------"
write(*,*) "Processing snapshot ", snap
write(*,*) "---------------------------------"
write(*,*) "Resolution:      ", resolution, "[dpi]"

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!

!Determine Total Number of Ranks for snapshots 
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

!Determine the total Number of ranks (nranks)
cmd = 'ls *snap'//snap//' > /tmp/snaplist_'//TRIM(pidstr(2:(pidlen+1)))
CALL SYSTEM(cmd)

OPEN(UNIT=1, FILE='/tmp/snaplist_'//TRIM(pidstr(2:(pidlen+1))))
nranks = 0
DO J=1,maxranks
  READ (UNIT=1, FMT=*, IOSTAT=ios) snapfile
  IF (ios < 0) EXIT
  nranks = nranks + 1
ENDDO
CLOSE(1)

WRITE(*,*) "Total number of ranks: ", nranks

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!

!Loop through Ranks and Read in data
!  and assign radius and theta value to each data point
!~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
OPEN(UNIT=1,FILE='/tmp/snaplist_'//TRIM(pidstr(2:(pidlen+1))))
count = 1
DO J=1,nranks
  READ(UNIT=1, FMT=*) snapfile
  WRITE(*,*) "reading in snapshot data from file: ", TRIM(snapfile)
  OPEN(UNIT=2, FILE=snapfile)
    READ(2,*) iout
    READ(2,*) dt
    READ(2,*) it
    READ(2,*) nz
    READ(2,*) minrad
    READ(2,*) maxrad
    READ(2,*) nx
    READ(2,*) mintheta
    READ(2,*) maxtheta
    IF (J == 1) THEN
      ALLOCATE(data(3,nranks*nx*nz))
    ENDIF    
    DO K=0,(nx-1)
      DO M=0,(nz-1)
        dr = ((maxrad-minrad)/(nz-1))/1000.
        radius = (maxrad - float(M)*((maxrad-minrad)/(nz-1)))/1000.

        !temp needs full testing  !!!Won't work if depth to cmb isn't 2891!!!
         dr = 2891./(float(nz-1))

        !check epsilon value of dtheta and redefine if >=
        ! |0.0001|
        IF (J == 1 .and. K == 1 .and. M == 1) THEN
          dtheta = ((maxtheta-mintheta)/(nx-1))*180./pi	
          nnnx = NINT((360.0/dtheta))
          epsilon = 360.0/dtheta - float(nnnx) 
          IF (ABS(epsilon) >= 0.0001) THEN
            dtheta = 360.0_8/(float(nnnx))
          ENDIF
        ENDIF

        theta = mintheta + float(K)*((maxtheta-mintheta)/(nx-1))
        data(1,count) = radius
        data(2,count) = theta*180./pi
        READ(2,*) data(3,count) 
        count = count + 1
      ENDDO
    ENDDO
  CLOSE(2)
ENDDO
CLOSE(1)

!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Non-linear scaling
write(*,*) "scaling data ..."
write(*,*) MAXVAL(ABS(data(3,:)))
data(3,:) = data(3,:)/MAXVAL(ABS(data(3,:)))
DO J=1,(count-1)
  !Gunnar's style of scaling
  B = data(3,J)
  A = (ABS(data(3,J)))**scale
  data(3,J) = SIGN(A,B)
ENDDO
!data(3,:) = data(3,:)/MAXVAL(ABS(data(3,:)))


!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++!
! Write out of data 
write(*,*) "writing data ..."

!determine proper rotation
rotation = (mina + (maxa-mina)/2.) - 90.0
write(*,*) "Rotation= ", rotation

!calculate width, height, xoffset, and yoffset
  IF ((maxa-mina) <= 180.0) THEN
    arg = ((maxa-mina)/2.0)*(3.141592654/180.0)
    xmax = maxr*SIN(arg) 
    !this is assuming that for postscript the total width is 9 inches
    width = 9.0
    tscale = xmax/(width/2.0)
    ymin = minr*COS(arg)
    height = (maxr - ymin)/tscale
    IF (height > 6.75) THEN
      tscale = 6.75/height
      height = 6.75
      width = width*tscale
    ENDIF
    xoffset = (9.333 - width)/2.0
    yoffset = (7.0 - height)/2.0
  ELSE
    width=9.0
    xmax = 6371.0*2.0
    tscale = xmax/width
    ymax = 6371.0 
    arg = ((360.0 - (maxa-mina))/2.0)*(3.141592654/180.0)
    ymin = maxr*COS(arg)
    height = (ymax+ymin)/tscale
    IF (height > 6.75) THEN
      tscale = 6.75/height
      height = 6.75
      width = width*tscale
    ENDIF
    xoffset=(9.333-width)/2.0
    yoffset=(7.0-height)/2.0
  ENDIF

  write(*,*) "width= ", width
  write(*,*) "height= ", height
  write(*,*) "xoffset= ", xoffset
  write(*,*) "yoffset= ", yoffset


OPEN(UNIT=3, FILE='.plotinfo')
write(3,*) dr
write(3,100) dtheta
write(3,*) mina
write(3,*) maxa
write(3,*) minr
write(3,*) maxr
write(3,*) rotation
write(3,*) NINT(float(it)*dt)
write(3,*) snap
write(3,*) otype
write(3,*) resolution
write(3,*) '/tmp/snapdata_'//TRIM(pidstr(2:(pidlen+1)))
write(3,*) width
write(3,*) xoffset
write(3,*) height
write(3,*) yoffset
CLOSE(3)

100 FORMAT(F16.13)

!Write output data in binary format
OPEN(UNIT=4,FILE='/tmp/snapdata_'//TRIM(pidstr(2:(pidlen+1))),FORM='unformatted')
DO J=1,(count-1)
  IF (data(1,J) <= 6371.0 .and. data(1,J) >= 3480.0) THEN
    IF (data(2,J) >= 0.0 .and. data(2,J) <= 180.0) THEN
      write(4) data(2,J), data(1,J), data(3,J)
      write(4) -data(2,J), data(1,J), data(3,J)
    ENDIF
  ENDIF
ENDDO
CLOSE(4)

!write discontinuities 410 and 660
write(*,*) "writing discontinuities ..."
OPEN(UNIT=5,FILE='/tmp/410disc')
OPEN(UNIT=6,FILE='/tmp/660disc')
OPEN(UNIT=7,FILE='/tmp/surfdisc')
OPEN(UNIT=8,FILE='/tmp/cmbdisc')
OPEN(UNIT=9,FILE='/tmp/laydisc')
DO J=-180,180
  write(5,*) J, 5961
  write(6,*) J, 5711
  write(7,*) J, 6371
  write(8,*) J, 3480
  write(9,*) J, 3744
ENDDO
CLOSE(5)
CLOSE(6)
CLOSE(7)
CLOSE(8)
CLOSE(9)

write(*,*) "generating plot ..."
cmd = './gmtscript.csh'
CALL SYSTEM(cmd)

cmd = 'rm /tmp/snaplist_'//TRIM(pidstr(2:(pidlen+1)))
CALL SYSTEM(cmd)

END PROGRAM shsnaps
